import re

class DummyStat(dict):
    def __getitem__(self, kname):
        if kname == "host":
            return "N/A"
        else:
            return 0
    def has_key(self, kname):
        return True

DUMMY_STAT = DummyStat()

def abbrevStat(sname):
    s = sname.replace("VmCpu", "")
    s = s.replace("Pct", "")
    return s

def load_resfile(filename):
    lst = []
    f = open(filename, "r")
    for l in f.readlines():
        if l.find("#") == 0:
            continue
        thisline = l.split()
        if thisline[0] == "hostaffin":
            # TODO: report affinity somewhere?
            continue
        elif thisline[0] == "antiaffin":
            continue
        elif thisline[0] == "affin":
            continue
        typ, name, parent, cshares, cmin, cmax, mshares, mmin, mmax = thisline[0:9]
        if cmax == "unlimited":
            cmax = -1
        if mmax == "unlimited":
            mmax = -1
        mem = Resource(int(mmin), int(mmax), int(mshares), -1)
        cpu = Resource(int(cmin), int(cmax), int(cshares), -1)
        if typ == "group":
            lst.append(Group(name, parent, cpu, mem))
        else:
            lst.append(VM(name, parent, cpu, mem))
    f.close()
    return lst

class Move:
    def __init__(self, vm, src, dest, pnum, avg=0.0, peak=0.0, foraffin=False):
        self.vm = vm
        self.src = src
        self.dest = dest
        self.passnum = pnum
        self.avgGood = avg
        self.peakGood = peak
        self.affin = foraffin
        
class Resource:
    def __init__(self, amin, amax, ashares, base):
        self.amin = amin
        self.amax = amax
        self.ashares = ashares
        self.base = base


class Node:
    def __init__(self, name, parent, cpu, mem):
        self.name = name
        self.parent = parent
        self.rsc = { "cpu" : cpu,
                     "mem" : mem }

class StatEntity:
    def __init__(self, name):
        self.name = name
        self.stats = { }

    def GetStatMinMax(self, stype, valtype):
        slist = self.stats[stype]
        l = [v[valtype] for v in slist]
        return min(l), max(l)

    def AddStat(self, val, stype, passnum):
        if self.stats.has_key(stype):
            slist = self.stats[stype]
            missingvals = passnum - len(slist) - 1
            # assert missingvals >= 0, 'p=%d l=%d' % (passnum, len(slist))
            if missingvals > 0:
                dummies = [DUMMY_STAT for x in range(missingvals)]
                slist.extend(dummies)
            self.stats[stype].append(val)
        else:
            self.stats[stype] = [val]
        
class Host(StatEntity):
    def __init__(self, name):
        StatEntity.__init__(self, name)
        # TODO: read this for real
        self.numcpus = 2

class VM(StatEntity, Node):
    namere = re.compile('\D+(\d+)')
    def __init__(self, name, parent, cpu, mem):
        StatEntity.__init__(self, name)
        Node.__init__(self, name, parent, cpu, mem)
        # TODO: read this for real
        self.numcpus = 1

    def GetShortName(self):
        m = VM.namere.search(self.name)
        return 'vm' + m.group(1)

class Group(StatEntity, Node):
    def __init__(self, name, parent, cpu, mem):
        StatEntity.__init__(self, name)
        Node.__init__(self, name, parent, cpu, mem)
        # TODO: sum over num members
        self.numcpus = 4

class Parser:
    def Limits_NormEnt(self, node):
        return 0.0, 1.8

    def Limits_Mhz(self, node):
        roundghz = round(.5 + (500 + self.maxmhz * node.numcpus) / 1000.0)
        return 0.0, roundghz * 1000

    def GetStatTypes(self):
        return self.stattypes

    def GetNodes(self, nodeType):
        if nodeType == "vms":
            return self.vms
        elif nodeType == "groups":
            return self.groups
        elif nodeType == "hosts":
            return self.hosts
        else:
            return None

    def GetValTypes(self, etype):
        # TODO: take entity type into account
        # and only return relevant valtypes
        if etype == "group" or etype == "vm":
            return ["cbase", "cEntitled", "active", "run", "host"]
        else:
            return ["capacity", "entitled", "cpuNormEnt", "memNormEnt"]
    
    def __init__(self, logf, resf):
        self.logFname = logf
        self.resFname = resf

        # name->Node mappings
        self.vms = {}
        self.hosts = {}
        self.groups = {}
        
        self.maxmhz = 0
        self.maxpass = 0
        self.stattypes = []
        self.moves = []
        
        # maps valtype to info about that type
        self.metadata = {
            "cbase" : ("mhz", self.Limits_Mhz),
            "cEntitled" : ("mhz", self.Limits_Mhz),
            "active" : ("mhz", self.Limits_Mhz),
            "run" : ("mhz", self.Limits_Mhz),
            "capacity" : ("mhz", self.Limits_Mhz),
            "entitled" : ("mhz", self.Limits_Mhz),
            "cpuNormEnt" : ("normEnt", self.Limits_NormEnt),
            "memNormEnt" : ("normEnt", self.Limits_NormEnt)
            }

        # regular expressions to parse drm logfiles
        anyregstr = '\[DrmState\]\s+(\S+)'
        groupregstr = anyregstr + "\s\((.*)\)\s+cbase:\s+(\d+)\s+" + \
                 "cEntitled:\s+(\d+)\s+mentitled:\s+(\d+)"
        vmregstr = groupregstr + "\s+active:\s+(\d+)\s+run: (\d+)"
        hostregstr= anyregstr + "\s+\(\s+(\S+)\)\s+mhzCapacity:\s+(\d+)\s+" \
                    + "mhzEntitled:\s+(\d+)\s+cpuNormEnt:\s+(\S+)\s+memNormEnt:\s+(\S+)" \
                    + "\s+cpuComp:\s+(\S+)\s+expMhz:\s+(\S+)"
        moveregstr = '\[DrmMoves\] mod-rec move: vm:(\S+) from (\S+) to (\S+), avgGoodness=(\S+), peakGoodness=(\S+)'
        affinmoveregstr = '\[DrmMoves\] - affinmove: (?:vm:)?(?:agroup with )?(\S+) from (\S+) to (\S+), avgGoodness=(\S+), peakGoodness=(\S+)'

        self.anyreg = re.compile(anyregstr)
        self.groupreg = re.compile(groupregstr)
        self.vmreg = re.compile(vmregstr)
        self.hostreg = re.compile(hostregstr)
        self.passreg = re.compile("\*+ pass (\d+)")
        self.movereg = re.compile(moveregstr)
        self.affinmovereg = re.compile(affinmoveregstr)

        self.LoadResFile()

    def GetYLabel(self, node, stattype, valtype):
        if self.metadata.has_key(valtype):
            desc, limcalc = self.metadata[valtype]
            return desc
        else:
            return ""
    
    def GetYLim(self, node, stattype, valtype):
        # ignore stattype for now
        if self.metadata.has_key(valtype):
            desc, limcalc = self.metadata[valtype]
            return limcalc(node)
        else:
            return node.GetStatMinMax(stattype, valtype)
        
    def LoadVMStat(self, line, passnum, stattype, host):
        m = self.vmreg.search(line)
        if not m:
            print "failed to load vm from: ", line
            return
        
        typ = m.group(1)
        name = m.group(2).strip()
        run = int(m.group(7))
        ent = int(m.group(4))
        if ent > 0:
            fair = float(run) / ent
        else:
            fair = 0.0
        newstat = dict(cbase = int(m.group(3)),
             cEntitled = ent,
             mtarget = int(m.group(5)),
             active = int(m.group(6)),
             run = run,
             fairness = fair,          
             host = host)
        thisv = None
        thisv = self.vms[name]
        thisv.AddStat(newstat, stattype, passnum)
        return thisv

    def LoadGroup(self, line, passnum, curstat):
        pass

    def LoadHost(self, line, passnum, stattype):
        m = self.hostreg.search(line)
        if not m:
            print "failed to load host from: ", line
            return
        
        typ = m.group(1)
        name = m.group(2).strip()
        newstat = dict(capacity = int(m.group(3)),
                       entitled = int(m.group(4)),
                       cpuNormEnt = float(m.group(5)),
                       memNormEnt = float(m.group(6)),
                       cpuComp = float(m.group(7)),
                       cpuExp = float(m.group(8)))
        thish = None
        if self.hosts.has_key(name):
            thish = self.hosts[name]
        else:
            thish = Host(name)
            self.hosts[name] = thish
            # see if we need to increase maxmhz (TODO: numPcpus?)
            thismhz = newstat["capacity"] / thish.numcpus
            if thismhz > self.maxmhz:
                self.maxmhz = thismhz
        thish.AddStat(newstat, stattype, passnum)
        return thish

    def GetHostNames(self):
        return self.hosts.keys()

    def GetNodeList(self, ntype):
        if ntype == "vm":
            return self.vms
        elif ntype == "group":
            return self.groups
        elif ntype == "host":
            return self.hosts
        else:
            return None
    
    def GetNodeNames(self, ntype):
        nlist = self.GetNodeList(ntype)
        if not nlist:
            return []
        else:
            return nlist.keys()

    def GetNode(self, typ, name):
        nlist = self.GetNodeList(typ)
        return nlist[name]
        
        
    def GetHistory(self, node, stattype, valtype):
        statvals = node.stats[stattype]
        history = [v[valtype] for v in statvals]
        return history

    def GetVM(self, name):
        return self.vms[name]

    def GetMaxPassnum(self):
        return self.maxpass
    
    def GetHost(self, name):
        return self.hosts[name]

    def LoadResFile(self):
        nodes = load_resfile(self.resFname)
        for n in nodes:
            if n.__class__ is VM:
                self.vms[n.name] = n
            else:
                self.groups[n.name] = n

    def GetMoves(self):
        return self.moves

    def LoadDrmFile(self):
        f = open(self.logFname, "r")
        passnum = 0
        curstat = None
        prevhost = ""
        seenHosts = 0
        self.hlist = []
        for l in f.xreadlines():
            # figure out which pass we're on by the "**** pass XX ****" lines
            passm = self.passreg.search(l)
            if passm:
                passnum = int(passm.group(1))
                seenHosts = 0 # reset number seen in this pass
            aff = False
            m = self.movereg.search(l)
            if not m:
                # see if it's an affinity-driven move
                m = self.affinmovereg.search(l)
                aff = True
            if m:
                vm = m.group(1)
                srcH = m.group(2)
                destH = m.group(3)
                avggood = float(m.group(4))
                peakgood = float(m.group(5))
                print 'vm %s moved to host %s in pass %d' % (vm, destH, passnum)
                self.moves.append(Move(self.GetVM(vm), srcH, destH,
                                       passnum, avg=avggood, peak=peakgood,foraffin=aff))
            m = self.anyreg.search(l)
            if m:
                typ = m.group(1)
                if typ == "vm":
                    self.LoadVMStat(l, passnum, curstat, prevhost)
                elif typ == "group":
                    self.LoadGroup(l, passnum, curstat)
                elif typ == "host":
                    h = self.LoadHost(l, passnum, curstat)
                    prevhost = h.name
                    seenHosts += 1
                    if h.name not in self.hlist:
                        self.hlist.append(h.name)
                elif typ == "stat":
                    newstat = l.split()[2]
                    newstat = abbrevStat(newstat)
                    curstat = newstat
                    if not newstat in self.stattypes:
                        print "stattype: ", newstat
                        self.stattypes.append(newstat)

        if seenHosts > 0:
            self.maxpass = passnum
        else:
            self.maxpass = passnum - 1
        f.close()



if __name__ == "__main__":
    assert False, "currently broken, run plotframe instead"
    p = Parser("drmoutput.txt", "resources.txt")
    p.LoadDrmFile()
    for n in p.GetNodeNames("vm"):
        print n
    v = p.GetNode("vm", 'linup2')
    print p.GetHistory(v, "ActAvg1Min", "run")
    h = p.GetHost('fake1')
    for v in p.GetHistory(h, "ActAvg1Min", "cpuNormEnt"):
        print "%1.2f " % (v),
               
