import drmparse, re, types
import matplotlib
matplotlib.use('WXAgg')
# from pylab import *
from matplotlib.figure import Figure
from matplotlib.axes import Subplot
from matplotlib.numerix import *
from pylab import show

from time import time

prevtime = time()
def TRACE(s=""):
    global prevtime
    t = time()
    print s, "%2.3f" % (t-prevtime)
    prevtime = t

import drmparse

class PlotElement:
    defaultStyles = ['b-', 'r--', 'c:', 'm+']
    def __init__(self, node, stypes, vtypes, styles=None):
        self.node = node
        self.valtypes = vtypes

        if type(stypes) is not types.ListType:
            self.stattypes = [stypes for v in self.valtypes]
        else:
            self.stattypes = stypes

        assert len(self.valtypes) == len(self.stattypes)
        if styles is not None:
            assert len(styles) == len(self.stattypes)
            self.styles = styles
        else:
            self.styles = []
            for r in range(len(self.stattypes)):
                defs = PlotElement.defaultStyles
                self.styles.append(defs[r % len(defs)])

class Plotter(Figure):
    def vmplot(self, pelem, cursubplot):
        cursubplot.set_title(pelem.node.name)
        p = None
        for stat,val,style in zip(pelem.stattypes,pelem.valtypes,pelem.styles):
            mystats = pelem.node.stats[stat]
            statlst = [x[val] for x in mystats]
            statarr = array(statlst)
            xseries = arange(len(statlst))
            p = cursubplot.plot(xseries, statarr, style, label=val)
            if self.wantLegend:
                cursubplot.legend()
            
        mylim = self.parser.GetYLim(pelem.node, pelem.stattypes[0], pelem.valtypes[0])
        cursubplot.set_ylim(mylim)
        cursubplot.set_xlim( (0, self.parser.GetMaxPassnum()) )
        mylabel = self.parser.GetYLabel(pelem.node, pelem.stattypes[0], pelem.valtypes[0])
        cursubplot.set_ylabel(mylabel)
        return p

    def GetSelectedPassnum(self):
        return self.selectedpos

    def plotall(self, wantLegend=True):
        self.wantLegend = wantLegend
        elemList = self.elemlist
        numplots = len(elemList)
        for i in range(numplots):
            pelem = elemList[i]
            cursubplot = self.add_subplot(numplots, 1, i+1)
            retplot = self.vmplot(pelem, cursubplot)
            if i < numplots-1:
                cursubplot.set_xticklabels([])
            else:
                ticklocs = cursubplot.get_xticks()
                ticknames = [str(int(x+1)) for x in ticklocs]
                cursubplot.set_xticklabels(ticknames)

            self.allplots.append( (cursubplot, pelem) )

    def plotmig(self, move):
        if move.passnum == 1:
            # moves in first pass not actually taken
            # as that's just the setup pass
            return
        
        for plt, pelem in self.allplots:
            vstats = move.vm.stats[pelem.stattypes[0]]
            # our mapping from passnums to data coords is a
            # bit screwed up. "dataCoord + 1 = passnum"
            # we want the label to appear just after the pass tick,
            # so subtract slightly less than one from the passnum
            moveoffset = - 0.95
            if isinstance(pelem.node, drmparse.Host):
                hname = pelem.node.name.lower()
                if hname == move.dest.lower():
                    plt.text(move.passnum + moveoffset, 0.1,
                             move.vm.GetShortName(),
                             rotation=90,
                             color='blue')                    
                elif move.src.lower() == hname:
                    # print 'src=',hname
                    mycol = 'gray'
                    if move.affin:
                        mycol = 'orange'
                    plt.text(move.passnum + moveoffset, 0.1, move.vm.GetShortName(),
                             rotation=90,
                             fontstyle='italic',
                             color=mycol)
            elif isinstance(pelem.node, drmparse.VM):
                vname = pelem.node.name.lower()
                if vname == move.vm.name.lower():
                    # see that this move actually worked
                    if move.passnum+1 < self.parser.GetMaxPassnum() and \
                       vstats[move.passnum+1]['host'].lower() == move.dest.lower():
                        plt.text(move.passnum + 1.0 + moveoffset, 1500,
                                 move.dest,
                                 rotation=90,
                                 color='black')
                        

    def HandleClick(self, event):
        ax = event.inaxes
        if not ax:
            return
        
        if event.button == 1:
            self.selectedpos = int(round(event.xdata))
            # left click: show the vertical line at the click point
            for plt, pelem in self.allplots:
                if self.indicatorlines.has_key(plt):
                    oldline = self.indicatorlines[plt]
                    oldline.set_xdata([self.selectedpos, self.selectedpos])
                else:
                    self.indicatorlines[plt] = plt.axvline(self.selectedpos)
                    mylim = self.parser.GetYLim(pelem.node,
                                                pelem.stattypes[0],
                                                pelem.valtypes[0])
                    plt.set_ylim(mylim)
                    plt.set_xlim( (0, self.parser.GetMaxPassnum()) )
        # Caller must redraw!

    def __init__(self, elemlist, parser):
        Figure.__init__(self, (12,8))
        #connect('button_press_event', self.on_click)
        self.elemlist = elemlist
        self.indicatorlines = {}
        self.allplots = []
        self.selectedpos = -1
        self.parser = parser


if __name__ == "__main__":
    print "run plotframe.py instead"
        
