import re
import sys
import math

def abbrevStat(sname):
    s = sname.replace("VmCpu", "")
    s = s.replace("Pct", "")
    return s



class StatEntity:
    def __init__(self, name):
        self.name = name
        self.stats = {}

    def AddStat(self, val, stype):
        if self.stats.has_key(stype):
            slist = self.stats[stype]
            self.stats[stype].append(val)
        else:
            self.stats[stype] = [val]

    def GetPrevious(self, stype):
        if self.stats.has_key(stype):
           sizeList = len(self.stats[stype])
           return self.stats[stype][sizeList-1]
        else:
           return {}

    def GetStat(self, atPass, stype):
        if self.stats.has_key(stype):
           current = len(self.stats[stype]) - 1
           while (current>=0):
              currentPass =  self.stats[stype][current]["pass"]
              if currentPass == atPass:
                 return self.stats[stype][current]
              elif currentPass < atPass:
                 return {}
              else:
                 current -= 1
           return {}     
        else:
           return {}


class Host(StatEntity):
    def __init__(self, name):
        StatEntity.__init__(self, name)

class VM(StatEntity):
    def __init__(self, name, parent):
        StatEntity.__init__(self, name)
        



class Parser:


   def __init__(self, fileName, isSimulate=0, oldFormat=0):
        self.fileName = fileName
        self.oldFormat = oldFormat
        self.isSimulate = isSimulate 
        self.curPass = 0

        if oldFormat:  
           anyRegStr = "\[DrmState\]\s+(\S+)"
        else:
           anyRegStr = "\[Drm\]\s+(\S+)"

        if not isSimulate:
           anyRegStr = "\[.*\]\s+"+anyRegStr

        groupRegStr = anyRegStr + "\s\((.*)\)\s+cbase:\s+(\d+)\s+" + \
                 "cEntitled:\s+(\d+)"

        if oldFormat:
           vmRegStr = groupRegStr + "\s+active:\s+(\d+)\s+run:\s+(\d+)\s" \
                      +"+cMissed:\s+(\d+)\s+mEntitled:" \
                      +"\s+(\d+)\s+current:\s+(\d+)\s+active:\s+(\d+)\s"\
                      +"+zeroShared:\s+(\d+)" \
                      +"\s+ballooned:\s+(\d+)\s+swapped:\s+(\d+)"
        else:
            vmRegStr = groupRegStr + "\s+active:\s+(\d+)\s+run:\s+(\d+)\s" \
                      +"+cMissed:\s+(\d+)\s+mbase:\s+(\d+)\s+mEntitled:" \
                      +"\s+(\d+)\s+current:\s+(\d+)\s+active:\s+(\d+)\s"\
                      +"+zeroShared:\s+(\d+)" \
                      +"\s+ballooned:\s+(\d+)\s+swapped:\s+(\d+)"
     
        if oldFormat:  
           hostRegStr= anyRegStr + "\s+\((\S+)\)\s+mhzCapacity:\s+(\d+)\s+" \
                    + "mhzEntitled:\s+(\d+)\s+cpuNormEnt:\s+(\S+)\s" \
                    +"+memNormEnt:\s+(\S+)" \
                    + "\s+cpuComp:\s+(\S+)\s+expMhz:\s+(\S+)"
        else:       
           hostRegStr= anyRegStr + "\s+\((\s*\S+)\)\s+mhzCapacity:\s+(\d+)\s+" \
                    + "mhzEntitled:\s+(\d+)\s+cpuNormEnt:\s+(\S+)\s+" \
                    + "mbCapacity:\s+(\d+)\s+memNormEnt:\s+(\S+)" \
                    + "\s+cpuComp:\s+(\S+)\s+expMhz:\s+(\S+)"
      
        if isSimulate:
           moveStartRegStr = "moving vm (\S+) from host (\S+) to (\S+) in pass (\d+)"
           moveFinishRegStr = moveStartRegStr
           passRegStr = "\*\*\*\*\*\*\*\*\*\*\*\*\*\*\* pass (\d+) \(time (\d+)\)"\
                        +" \*\*\*\*\*\*\*\*\*\*\*\*\*\*\*"
        else:
           moveStartRegStr = "\[.*\]\s+\[EventManagerMo\] "\
                            +"Event\[\d+\]: Migrating (\S+) from (\S+) to (\S+)" 
           moveFinishRegStr = "\[.*\]\s+\[EventManagerMo\] Event\[\d+\]:"\
                             +" DRS migrated (\S+) from (\S+) to (\S+)"
           passRegStr = "\[.*\]\s+\[Drm\] Drmdump written:"
           self.curPass = 1
 
        self.anyReg = re.compile(anyRegStr)
        self.groupReg = re.compile(groupRegStr)
        self.vmReg = re.compile(vmRegStr)
        self.hostReg = re.compile(hostRegStr)
        self.moveStartReg = re.compile(moveStartRegStr)
        self.moveFinishReg = re.compile(moveFinishRegStr)
        self.passReg = re.compile(passRegStr)
        # name->Node mappings
        self.vms = {}
        self.hosts = {}
        self.groups = {}
        self.statTypes = []
        self.perPassStats = {}

   def LoadGroup(self, line, statType):
        #Currently group stats are not required
        return

   def LoadHost(self, line, statType):
        m = self.hostReg.search(line)
        if not m:
            print "failed to load host from: ", line
            return
        typ = m.group(1)
        name = m.group(2).strip()
        if self.oldFormat:
           newStat = { "capacity":int(m.group(3)),
                       "entitled":int(m.group(4)),
                       "cpuNormEnt":float(m.group(5)),
                       "memNormEnt":float(m.group(6)),
                       "cpuComp":int(m.group(7)),
                       "cpuExp":int(m.group(8)),
                       "pass":self.curPass
                     }
        else: 
           newStat = { "capacity":int(m.group(3)),
                       "entitled":int(m.group(4)),
                       "cpuNormEnt":float(m.group(5)),
                       "memNormEnt":float(m.group(7)),
                       "cpuComp":int(m.group(8)),
                       "cpuExp":int(m.group(9)),
                       "pass":self.curPass
                     }
        thish = None
        if self.hosts.has_key(name):
            thish = self.hosts[name]
        else:
            thish = Host(name)
            self.hosts[name] = thish
        thish.AddStat(newStat, statType)
        return thish



   def LoadVM(self, line, statType, host):
        m = self.vmReg.search(line)
        if not m:
            print "failed to load vm from: ", line
            return
        
        typ = m.group(1)
        name = m.group(2).strip()
        run = int(m.group(6))
        ent = int(m.group(4))
        if ent > 0:
            fair = float(run) / ent
        else:
            fair = 0.0

        if self.vms.has_key(name):
            thisVm = self.vms[name]
            previousStat = thisVm.GetPrevious(statType)
        else:
            thisVm = VM(name, host)
            self.vms[name] = thisVm
            previousStat = {} 

        if self.oldFormat:
           current = int(m.group(9))
           zeroShared = int(m.group(11))
        else:
           current = int(m.group(10))
           zeroShared = int(m.group(12))

        mLoad = current - zeroShared
        if mLoad < 0:
           mLoad = 0

        if len(previousStat) > 0:
           mDynamicShares = previousStat["mEntitled"]
           cDynamicShares = previousStat["cEntitled"]
           cStaticShares =  previousStat["cbase"]
           cLoad = run
           if mDynamicShares > 0:
              mDynamicLoadFair = float(mLoad)/float(mDynamicShares)
           else:
              mDynamicLoadFair = 0.0

           if cDynamicShares > 0:
              cDynamicLoadFair = float(cLoad)/float(cDynamicShares)
           else:
              cDynamicLoadFair = 0.0
        else:
           cDynamicLoadFair = -1.0
           mDynamicLoadFair = -1.0

        if self.oldFormat:
           newStat = {"cbase":int(m.group(3)),
                      "cEntitled":ent,
                      "mEntitled":int(m.group(8)),
                      "active":int(m.group(5)),
                      "run":run,
                      "current":current,
                      "zeroShared":zeroShared, 
                      "balloned":int(m.group(12)),
                      "swapped":int(m.group(13)),
                      "fairness":fair,          
                      "host":host,
                      "mLoadFair":mDynamicLoadFair,
                      "cLoadFair":cDynamicLoadFair,
                      "pass":self.curPass}
        else: 
            newStat = {"cbase":int(m.group(3)),
                      "cEntitled":ent,
                      "mEntitled":int(m.group(9)),
                      "active":int(m.group(5)),
                      "run":run,
                      "current":current,
                      "zeroShared":zeroShared, 
                      "balloned":int(m.group(13)),
                      "swapped":int(m.group(14)),
                      "fairness":fair,          
                      "host":host,
                      "mLoadFair":mDynamicLoadFair,
                      "cLoadFair":cDynamicLoadFair,
                      "pass":self.curPass}

        if (self.displayNext.has_key(name)):
           self.displayNext[name].append(newStat)

        thisVm.AddStat(newStat, statType)
        return name



   def  ComputeClusterFairness(self, numMig, curStat):
        atPass = self.curPass
        vmMemFairTot = 0.0
        vmCpuFairTot = 0.0
        vmMemSqr = 0.0
        vmCpuSqr = 0.0
        numVms = 0.0
        print "********************** Pass : %d **************"%(atPass)
        for vm in self.vms:
           vmstats = self.vms[vm].GetStat(atPass, curStat)
           if len(vmstats) > 0:
              mFair = vmstats["mLoadFair"] 
              cFair = vmstats["cLoadFair"]
              if (mFair > -1) and (cFair > -1):
                 vmMemFairTot += mFair
                 vmCpuFairTot += cFair
                 vmMemSqr += (mFair * mFair)
                 vmCpuSqr += (cFair * cFair)
                 numVms += 1
                 memLoad = vmstats["current"]-vmstats["zeroShared"]
                 print " ****%-10s  cLoadFair %3.4f  cLoad %5d  curSharesCpu"\
                              " %5d mLoadFair %3.4f  mLoad %5d  curSharesMem %5d"\
                              "  Balloned %d  ZeroShared %d Swapped %d"\
                        %(vm, cFair, vmstats["run"], vmstats["cEntitled"],\
                          mFair, memLoad, vmstats["mEntitled"], vmstats["balloned"],\
                          vmstats["zeroShared"], vmstats["swapped"] )
        vmMemSd = vmCpuSd = 0.0
        vmMemAvg = vmCpuAvg = 0.0 
        if numVms > 1:
           vmMemSd = math.sqrt((1.0 / (numVms - 1.0)) * ( vmMemSqr - \
                                  ((vmMemFairTot * vmMemFairTot) / numVms)))
           vmCpuSd = math.sqrt((1.0 / (numVms - 1.0)) * ( vmCpuSqr - \
                                  ((vmCpuFairTot * vmCpuFairTot) / numVms)))
        if numVms > 0:
           vmMemAvg = vmMemFairTot / numVms
           vmCpuAvg = vmCpuFairTot / numVms

        newStat = {"vmMemSd":vmMemSd,
                   "vmCpuSd":vmCpuSd}
        self.perPassStats[atPass] =  newStat
        print "Pass: %d, MemSd %f, CpuSd %f, numMigs %d mAvg %f  cAvg %f"%\
               (atPass, vmMemSd, vmCpuSd, numMig, vmMemAvg, vmCpuAvg)
        self.perPassPlot.write("%d, %f, %f, %d, %f, %f\n"%\
               (atPass, vmMemSd, vmCpuSd, numMig, vmMemAvg, vmCpuAvg))


   def  LoadVpxdFile(self):
        f = open(self.fileName, "r")
        curStat = None
        prevHost = ""
        seenHosts = 0
        numMigThisPass = 0
        self.hlist = []
        self.displayNext = {}
        migrationInProgress = {}
        migrationList = []
        self.perPassPlot = open("vpxdplot.dat","w")
        prevDump = ''
        for l in f.xreadlines():
            m = self.passReg.search(l)
            if m:
               self.ComputeClusterFairness(numMigThisPass, curStat)
               print prevDump
               prevDump = l
               self.curPass += 1
               numMigThisPass = 0
               continue
   
            m = self.moveStartReg.search(l) 
            if m:
               vmName = m.group(1)
               hostFrm = m.group(2)
               hostTo = m.group(3)
               if self.vms.has_key(vmName):
                  thisVm = self.vms[vmName]
                  prevStat = thisVm.GetPrevious(curStat)
               else:
                  prevStat = {}
                  prevStat["mLoadFair"] = -1.0
                  prevStat["cLoadFair"] = -1.0
               migrationInProgress[vmName] = [prevStat, hostFrm, hostTo, self.curPass]
               numMigThisPass += 1  

            m = self.moveFinishReg.search(l)
            if m:
               vmName = m.group(1)
               if migrationInProgress.has_key(vmName):
                  self.displayNext[vmName] = migrationInProgress[vmName]
                  del migrationInProgress[vmName]
               else:
                  #Do not print stats 
                  print "Migration ended for:"+vmName+" But cannot find starting message"
                  

            m = self.anyReg.search(l)
            if m:
                typ = m.group(1)
                if typ == "vm":
                    vm = self.LoadVM(l, curStat, prevHost)
                    if self.displayNext.has_key(vm):
                       migrationList.append([vm,self.displayNext[vm]])
                       del self.displayNext[vm]
                elif typ == "group":
                    self.LoadGroup(l, curStat)
                elif typ == "host":
                    h = self.LoadHost(l, curStat)
                    prevhost = h.name
                    seenHosts += 1
                    if h.name not in self.hlist:
                        self.hlist.append(h.name)
                elif typ == "stat":
                    newStat = l.split()[2]
                    newStat = abbrevStat(newStat)
                    curStat = newStat
                    if not newStat in self.statTypes:
                        self.statTypes.append(newStat)

        f.close()
        self.perPassPlot.close()

        print "\n Migration Statistics: \n"
        for m in migrationList:
           atPass = m[1][3]
           beforeMig = m[1][0]
           afterMig = m[1][4]
           if self.perPassStats.has_key(atPass-1):
              bMemSd = self.perPassStats[atPass - 1]["vmMemSd"]
              bCpuSd = self.perPassStats[atPass - 1]["vmCpuSd"]
           else:
              bMemSd = 0.0
              bCpuSd = 0.0
           if self.perPassStats.has_key(atPass+1):
              aMemSd = self.perPassStats[atPass + 1]["vmMemSd"]
              aCpuSd = self.perPassStats[atPass + 1]["vmCpuSd"]
           else:
              aMemSd = 0.0
              aCpuSd = 0.0

           print "Migration of %s from %s to %s in pass %d :"%\
                 (m[0], m[1][1], m[1][2], atPass)
           print "CPU   Vm before:%f   Vm after:%f  Cl before:%f  Cl after:%f"%\
                 (beforeMig["cLoadFair"], afterMig["cLoadFair"], bCpuSd, aCpuSd)
           print "MEM   Vm before:%f   Vm after:%f  Cl before:%f  Cl after:%f"%\
                 (beforeMig["mLoadFair"], afterMig["mLoadFair"], bMemSd, aMemSd)
                  
           
if __name__ == "__main__":
    fileName = sys.argv[1]
    if fileName.find("drmoutput") > -1:
       p = Parser(sys.argv[1], isSimulate=1)
    else:
       p = Parser(sys.argv[1])

    p.LoadVpxdFile()

