class UnreadableFile(file):

    def __init__(self, fp, verbose=False):
        self._fp = fp
        self._buffer = []
        self._verbose = verbose

    def unreadline(self, line):
        if self._verbose:
            print "<<<", line
        self._buffer.append(line)

    def readline(self):
        if len(self._buffer):
            line = self._buffer.pop()
        else:
            line = self._fp.readline().rstrip()
        if self._verbose:
            print ">>>", line
        return line

    def readlines(self):
        res = self._buffer + self._fp.readlines()
        self._buffer = []
        return res

class TabBasedParser(object):

    def __init__(self, fp, verbose=False, *args, **kwargs):
        if not isinstance(fp, UnreadableFile):
            fp = UnreadableFile(fp, verbose)
        self._fp = fp

    def _parseLine(self, min_size=0):
        l = self._fp.readline()
        items = l.split("\t")

        if min_size > len(items):
            for _ in range(min_size - len(items)):
                items.append(None)
        return items

    def _unparseItemsLine(self, items):
        l = "\t".join(items)
        self._fp.unreadline(l)

    def accept(self, visitor):
        pass

    def _parseTokenVal(self, item, token):
        """
        Parses an item passed as "token:val" and returns the value
        """
        vars = item.split(":")
        vars[0] = vars[0].strip()
        assert vars[0] == token

        return vars[1].strip()

    def _parseInlineArray(self, token):
        """
        Parses a line similar to
        <token> <num> <num # of items>
        """
        items = self._parseLine()
        assert items[0] == token
        num = int(items[1])
        assert num == len(items) - 2
        return items[2:]

    def _parseArray(self, token):
        """
        Parses a line similar to
        <token> <num>
        <num # of items>
        Also handles two gotchas
        1. If num is not present, num is assumed to be zero
        2. If num is zero and if the next line is empty, it is skipped
        """
        items = self._parseLine()
        assert items[0] == token

        if len(items) == 1: # Gotcha #1
            return []

        num = int(items[1])
        line = self._parseLine()
        if len(line[0]) == 0: # Gotcha #2
            line = line[1:]
        assert num == len(line)

        return line
