from v43 import *
import v43 as base

class StringOptionsParser(base.StringOptionsParser):

    TOKEN = "DrmModule::_stringOpts"

class IntOptionsParser(base.IntOptionsParser):

    TOKEN = "DrmModule::_intOpts"

class EnumOptionsParser(base.EnumOptionsParser):

    TOKEN = "DrmModule::_enumOpts"

class OptionsParser(base.OptionsParser):

    STRING_OPTIONS_PARSER = StringOptionsParser
    INT_OPTIONS_PARSER = IntOptionsParser
    ENUM_OPTIONS_PARSER = EnumOptionsParser

class ObjectParser(base.ObjectParser):

    def _acceptHeader(self, visitor):
        token, id, path, name = self._parseLine()
        assert token == "DrmObject"

        visitor.visitId(id)
        visitor.visitPath(path)
        visitor.visitName(name)

        token, uuid = self._parseLine()
        assert token == "DrmObject::_uuid"
        visitor.visitUuid(uuid)

class HostParser(base.HostParser):

    OBJECT_PARSER = ObjectParser

    CAPACITIES = ["cpu", "mem", "sio", "disk"]

    def _acceptHeader(self, visitor):
        tok, vmotion, state, cores, threads, mhz, boot_time, shutdown_time, last_exit, cpu, mem, auto_level, fsr, wakeable, version, max_vms, max_vcpu, build, name = self._parseLine()
        assert tok == "DrmHost"

        visitor.visitInVmotion(vmotion)
        visitor.visitHostState(state)
        visitor.visitNumCpuCores(cores)
        visitor.visitNumCpuThreads(threads)
        visitor.visitCpuMhz(mhz)
        visitor.visitBootTime(boot_time)
        visitor.visitShutDownTime(shutdown_time)
        visitor.visitTimeSinceLastExitStandby(last_exit)
        visitor.visitCpu(cpu)
        visitor.visitMemory(mem)
        visitor.visitAutoLevel(auto_level)
        visitor.visitSupportsFSR(fsr)
        visitor.visitWakeableNotUsingWol(wakeable)
        visitor.visitVersion(version)
        visitor.visitMaxVms(max_vms)
        visitor.visitMaxVcpus(max_vcpu)
        visitor.visitBuild(build)
        visitor.visitName(name)

    def _acceptSections(self, visitor):
        token, number = self._parseLine()
        assert token == "DrmHost::_availPortsPerDvs"
        for _ in range(int(number)):
            num, name = self._parseLine()
            visitor.visitDvs(name, num)

        token, number = self._parseLine()
        assert token == "DrmHost::_availPortsPerSwitch"
        for _ in range(int(number)):
            num, name = self._parseLine()
            visitor.visitSwitch(name, num)

        token, number = self._parseLine()
        assert token == "DrmHost::_network2Switch"
        for _ in range(int(number)):
            sw, net = self._parseLine()
            visitor.visitNetworkSwitch(net, sw)

        for it in self._parseInlineArray("DrmHost::_connectedDatastores"):
            visitor.visitConnectedDatastore(it)

        items = self._parseLine()
        assert items[0] == "DrmCapacity"
        for c, it in zip(self.CAPACITIES, items[1:]):
            visitor.visitCapacity(c, it)

        v = visitor.visitObject()
        self.OBJECT_PARSER(self._fp).accept(v)

class ConfigHostsParser(base.ConfigHostsParser):

    HOST_PARSER = HostParser

class AllocParser(base.AllocParser):

    def _acceptHeader(self, visitor):
        (token,) = self._parseLine()
        assert token == "DrmAlloc"

        v = visitor.visitResourceConfigSpec()
        self.RESOURCE_CONFIG_SPEC_PARSER(self._fp).accept(v)

        token, empty, id = self._parseLine()
        visitor.visitEmpty(empty, id)

class NodeParser(base.NodeParser):

    OBJECT_PARSER = ObjectParser
    ALLOC_PARSER = AllocParser

    def _acceptHeader(self, visitor):
        token, id = self._parseLine()
        assert token == "DrmNode"
        visitor.visitId(id)

class VmParser(base.VmParser):

    NODE_PARSER = NodeParser
    ALLOC_PARSER = AllocParser

    def _acceptHeader(self, visitor):
        tok, level, dest, on, state, vcpus, mem, overhead, swapped, agent, template, disks = self._parseLine()
        assert tok == "DrmVm"

        visitor.visitAutomationLevel(level)
        visitor.visitDestHostId(dest)
        visitor.visitRunningOn(on)
        visitor.visitVmState(state)
        visitor.visitNumVcpus(vcpus)
        visitor.visitMem(mem)
        visitor.visitOverhead(overhead)
        visitor.visitSwappedOverhead(swapped)
        visitor.visitIsAgentVm(agent)
        visitor.visitIsTemplateVm(template)
        visitor.visitDisksAffined(disks)

        for it in self._parseInlineArray("DrmVm::_compatibleHosts"):
            visitor.visitCompatibleHost(it)

        for it in self._parseInlineArray("DrmVm::_compatibleHostsNoHardRules"):
            visitor.visitCompatibleHostNoHardRules(it)

        for it in self._parseInlineArray("DrmVm::_preferredHosts"):
            visitor.visitPreferedHost(it)

        for it in self._parseInlineArray("DrmVm::_incompatibleVMs"):
            visitor.visitIncompatibleVm(it)

        for it in self._parseInlineArray("DrmVm::_affineVMs"):
            visitor.visitAffineVm(it)

    def _acceptSections(self, visitor):
        v = visitor.visitAlloc()
        self.ALLOC_PARSER(self._fp).accept(v)

        token, number = self._parseLine()
        assert token == "DrmVm::_recentVmotions"
        for _ in range(int(number)):
            raise Exception("Not implemented: item in %s" % (token))

        token, number = self._parseLine()
        assert token == "DrmVm::_predictedOverheadMap"
        for _ in range(int(number)):
            x, y = self._parseLine()
            visitor.visitPredictedOverhead(x, y)

        token, number = self._parseLine()
        assert token == "DrmVm::_networks"
        for _ in range(int(number)):
            token, type, nports, name, device = self._parseLine()
            visitor.visitNetwork(type, nports, name, device)

        token, number = self._parseLine()
        assert token == "DrmVm::_ftInfo"
        for _ in range(int(number)):
            raise Exception("Not implemented: item in %s" % (token))

        token, number = self._parseLine()
        assert token == "DrmVm::_disks"
        for _ in range(int(number)):
            raise Exception("Not implemented: item in %s" % (token))

        v = visitor.visitNode()
        self.NODE_PARSER(self._fp).accept(v)

class ConfigVmsParser(base.ConfigVmsParser):

    VM_PARSER = VmParser

class GroupParser(base.GroupParser):

    NODE_PARSER = NodeParser
    ALLOC_PARSER = AllocParser

class ConfigGroupsParser(base.ConfigGroupsParser):

    NODE_PARSER = NodeParser
    GROUP_PARSER = GroupParser

class ConfigVmIdsParser(TabBasedParser):

    def accept(self, visitor):
        vals = self._parseLine()
        assert vals[0] == "DrmDatastore::_vmIds"
        if len(vals) > 1:
            nvmIds = int(vals[1])
            pass # TODO: visitor callback

        visitor.visitEnd()

class ConfigCorrelatedDSParser(TabBasedParser):

    def accept(self, visitor):
        for it in self._parseArray("DrmDatastore::_correlatedDatastores"):
            pass # TODO: visitor callback

        visitor.visitEnd()

class ConfigConnectedHostsParser(TabBasedParser):

    def accept(self, visitor):
        for it in self._parseArray("DrmDatastore::_connectedHosts"):
            pass # TODO: visitor callback

        visitor.visitEnd()

class ConfigHaMastersParser(TabBasedParser):

    def accept(self, visitor):
        for it in self._parseArray("DrmDatastore::_haMasters"):
            pass # TODO: visitor callback

        visitor.visitEnd()

class ConfigSupportedFeaturesParser(TabBasedParser):

    def accept(self, visitor):
        for it in self._parseArray("DrmDatastore::_supportedFeatures"):
            pass # TODO: visitor callback

        visitor.visitEnd()



class ConfigDatastoresParser(ArrayParser):

    TOKEN = "DrmDomainConfig::_datastores"

    OBJECT_PARSER = ObjectParser
    CONFIG_VMIDS_PARSER = ConfigVmIdsParser
    CONFIG_CORRELATED_DATASTORES_PARSER = ConfigCorrelatedDSParser
    CONFIG_CONNECTED_HOSTS_PARSER = ConfigConnectedHostsParser
    CONFIG_HA_MASTERS_PARSER = ConfigHaMastersParser
    CONFIG_SUPPORTED_FEATURES_PARSER = ConfigSupportedFeaturesParser

    def _acceptHeader(self, visitor):
        items = self._parseLine()
        token = items[0]
        dsOptions = items[1:]

        assert token == "DrmDatastore"
        visitor.visitDatastoreOptions(dsOptions)

    def _visitItem(self, visitor):

        self._acceptHeader(visitor)

        visitor.visitVmIds()
        self.CONFIG_VMIDS_PARSER(self._fp).accept(visitor)

        visitor.visitCorrelatedDataStores()
        self.CONFIG_CORRELATED_DATASTORES_PARSER(self._fp).accept(visitor)

        visitor.visitConnectedHosts()
        self.CONFIG_CONNECTED_HOSTS_PARSER(self._fp).accept(visitor)

        visitor.visitHaMasters()
        self.CONFIG_HA_MASTERS_PARSER(self._fp).accept(visitor)

        visitor.visitSupportedFeatures()
        self.CONFIG_SUPPORTED_FEATURES_PARSER(self._fp).accept(visitor)

        visitor.visitObject()
        self.OBJECT_PARSER(self._fp).accept(visitor)

class ConfigParser(base.ConfigParser):

    CONFIG_HOSTS_PARSER = ConfigHostsParser
    CONFIG_VMS_PARSER = ConfigVmsParser
    CONFIG_GROUPS_PARSER = ConfigGroupsParser

    CONFIG_DATASTORES_PARSER = ConfigDatastoresParser

    def _acceptHeader(self, visitor):
        token, status, vmotion_rate, host_power_rate, root, timestamp = self._parseLine()
        assert token == "DrmDomainConfig"

        visitor.visitStatus(status)
        visitor.visitVmotionRate(vmotion_rate)
        visitor.visitHostPowerActionRate(host_power_rate)
        visitor.visitRoot(root)
        visitor.visitTimestamp(timestamp)

    def _acceptSections(self, visitor):
        base.ConfigParser._acceptSections(self, visitor)

        v = visitor.visitDatastores()
        self.CONFIG_DATASTORES_PARSER(self._fp).accept(v)

class RecentStatsParser(base.RecentStatsParser):

    def _acceptStats(self, visitor, n, typ):
        for _ in range(n):
            if typ == "0":
                token, timestamp = self._parseLine()
                assert token == "DrmStat"

                visitor.visitTimestamp(timestamp)
            elif typ == "1":
                token, timestamp, value = self._parseLine()
                assert token == "DrmStatFloat"

                visitor.visitTimestamp(timestamp)
                visitor.visitValue(value)
            elif typ == "2":
                token, timestamp, value = self._parseLine()
                assert token == "DrmStat64"

                visitor.visitTimestamp(timestamp)
                visitor.visitValue(value)

    def accept(self, visitor):
        token, size, timeout_sec, timeout, fake_time, number, typ = self._parseLine()
        assert token == "RecentStatsCache"

        visitor.visitMaxSize(size)
        visitor.visitTimeoutSec(timeout_sec)
        visitor.visitTimeout(timeout)
        visitor.visitFakeStatTime(fake_time)
        visitor.visitType(typ)

        self._acceptStats(visitor, int(number), typ)
        visitor.visitEnd()

class ObjectStatsParser(base.ObjectStatsParser):

    RECENT_STATS_PARSER = RecentStatsParser

    def accept(self, visitor):
        token, id, size, stats, number, timeout = self._parseLine()
        assert token == "DrmObjectStats:"

        visitor.visitId(id)
        visitor.visitSize(size)
        visitor.visitStatsAvailable(stats)
        visitor.visitTimeoutSec(timeout)

        self._acceptRecentStats(visitor, int(number))
        visitor.visitEnd()

class DatastoreStatsHistoryParser(TabBasedParser):

    def parseDSSamples(self):
        token, size = self._parseLine()
        assert token == "DrmDataStoreStats"

        samples = {}
        samples['data'] = []
        for _ in range(int(size)):
            token, percentile, oioR, oioW, latR, latW, \
            iopR, iosW, wR, wW, vmLat = self._parseLine()
            assert token == "DS"

            samples['data'].append({ \
                'percentile': float(self._parseTokenVal(percentile, 'Percentile')), \
                'oioR': float(self._parseTokenVal(oioR, 'oioR')), \
                'oioW': float(self._parseTokenVal(oioW, 'oioW')), \
                'latR': float(self._parseTokenVal(latR, 'latR')), \
                'latW': float(self._parseTokenVal(latW, 'latW')), \
                'iopR': float(self._parseTokenVal(iopR, 'iopR')), \
                'iosW': float(self._parseTokenVal(iosW, 'iosW')), \
                'wR': float(self._parseTokenVal(wR, 'wR')), \
                'wW': float(self._parseTokenVal(wW, 'wW')), \
                'vmLat': float(self._parseTokenVal(vmLat, 'vmLat')), \
            })

        slope, intercept = self._parseLine()
        samples['slope'] = float(self._parseTokenVal(slope, 'slope'))
        samples['intercept'] = float(self._parseTokenVal(intercept, 'intercept'))
        valid, day, duration, startTime, endTime = self._parseLine()
        return samples

    def accept(self, visitor):
        histid, token, size  = self._parseLine()
        assert token == "StatsHistory:"

        for _ in range(int(size)):
            idtoken, id, empty = self._parseLine()
            assert idtoken == "ID:"

            emptyval = self._parseTokenVal(empty, "empty")
            if int(emptyval) == 0:
                samples = self.parseDSSamples()
                visitor.visitDSSamples(histid, samples)
        visitor.visitEnd()


class DatastoreStatsParser(base.ArrayParser):

    TOKEN = "DrmDomainStats::_datastoreStats"
    DATASTORE_STATS_HISTORY_PARSER = DatastoreStatsHistoryParser

    def _visitItem(self, visitor):
        p = self.DATASTORE_STATS_HISTORY_PARSER(self._fp)
        p.accept(visitor)

class VmdkStatsHistoryParser(TabBasedParser):

    def parseVmdkSamples(self):
        token, size = self._parseLine()
        assert token == "DrmVmdkStoreStats"

        if size == 0:   return {}

        samples = {}
        samples['data'] = []
        for _ in range(int(size)):
            token, percentile, oioR, oioW, wR, wW, rKbps, wKbps = self._parseLine()
            assert token == "VMDK"

            samples['data'].append({ \
                'percentile': float(self._parseTokenVal(percentile, 'Percentile')), \
                'oioR': float(self._parseTokenVal(oioR, 'oioR')), \
                'oioW': float(self._parseTokenVal(oioW, 'oioW')), \
                'wR': float(self._parseTokenVal(wR, 'wR')), \
                'wW': float(self._parseTokenVal(wW, 'wW')), \
                'rKbps': float(self._parseTokenVal(rKbps, 'rKbps')), \
                'wKbps': float(self._parseTokenVal(wKbps, 'wKbps')), \
            })

        spaceAvg = self._parseLine()[0]
        samples['SpaceCommitedAverageRate'] =  \
            float(self._parseTokenVal(spaceAvg, 'SpaceCommittedAverageRate'))
        valid, day, duration, startTime, endTime = self._parseLine()
        return samples

    def accept(self, visitor):
        histid, token, size  = self._parseLine()
        assert token == "StatsHistory:"

        for _ in range(int(size)):
            idtoken, id, empty = self._parseLine()
            assert idtoken == "ID:"

            emptyval = self._parseTokenVal(empty, "empty")
            if int(emptyval) == 0:
                samples = self.parseVmdkSamples()
                visitor.visitVmdkSamples(histid, samples)
        visitor.visitEnd()


class VmdkStatsParser(base.ArrayParser):

    TOKEN = "DrmDomainStats::_vmdkStats"
    VMDK_STATS_HISTORY_PARSER = VmdkStatsHistoryParser

    def _visitItem(self, visitor):
        p = self.VMDK_STATS_HISTORY_PARSER(self._fp)
        p.accept(visitor)


class HostStatsParser(base.HostStatsParser):

    TOKEN = "DrmDomainStats::_hostStats"
    OBJECT_STATS_PARSER = ObjectStatsParser

    STATS_DESCRIPTOR = ["HostCpuAvailHist", "HostMemAvailHist",
                        "HostSysCpuAllocMin", "HostSysVmotionCpuAllocMin",
                        "HostVimCpuAllocMin", "HostSysMemAllocMin",
                        "HostSysVmotionMemAllocMin", "HostVimMemAllocMin",
                        "HostSamplePeriodSec"]

class VmStatsParser(base.VmStatsParser):

    STATS_DESCRIPTOR = ["VmCpuActAvg1MinPctHist", "VmCpuActAvg5MinPct",
                        "VmCpuActPeak5MinPct", "VmCpuRunAvg1MinPct",
                        "VmCpuRunAvg5MinPct", "VmCpuMaxLimited1MinPct",
                        "VmCpuMaxLimited5MinPct", "VmMemSharedZeroMB",
                        "VmMemSharedNonzeroMB", "VmMemGrantedMB",
                        "VmMemActiveMBHist", "VmMemActiveWriteMBHist",
                        "VmMemOverheadMB", "VmMemBalloonMB",
                        "VmMemSwapMB", "OptVmMemZippedMB",
                        "StatSampleCount", "OptVmMemOverheadGrowthRateHist",
                        "OptVmMemOvdBaseTimeInt64Hist", "OptVmMemOvhdComputeDurInt64Hist",
                        "VmSpaceCommittedMBHist", "VmSioRIopsHist",
                        "VmSioWIopsHist", "VmSioRKbpsHist",
                        "VmSioWKbpsHist", "VmSioRTotLatInMsHist",
                        "VmSioWTotLatInMsHist", "VmSioRDevLatInMsHist",
                        "VmSioWDevLatInMsHist", "VmSioROioHist",
                        "VmSioWOioHist"]

    TOKEN = "DrmDomainStats::_vmStats"
    OBJECT_STATS_PARSER = ObjectStatsParser

class StatsParser(base.StatsParser):

    HOST_STATS_PARSER = HostStatsParser
    VM_STATS_PARSER = VmStatsParser
    DATASTORE_STATS_PARSER = DatastoreStatsParser
    VMDK_STATS_PARSER = VmdkStatsParser
    RECENT_STATS_PARSER = RecentStatsParser

    def _acceptHeader(self, visitor):
        token, timeout, hist_size = self._parseLine()
        assert token == "DrmDomainStats"

        visitor.visitTimeout(timeout)
        visitor.visitMaxhistSize(hist_size)

    def _acceptSections(self, visitor):
        base.StatsParser._acceptSections(self, visitor)

        v = visitor.visitDatastoreStats()
        self.DATASTORE_STATS_PARSER(self._fp).accept(v)

        v = visitor.visitVmdkStats()
        self.VMDK_STATS_PARSER(self._fp).accept(v)

        v = visitor.visitRecentStats()
        self.RECENT_STATS_PARSER(self._fp).accept(v)

class RebalanceModuleParser(base.RebalanceModuleParser):

    OPTIONS_PARSER = OptionsParser
    SPI_PARSER = SpiParser
    CONFIG_PARSER = ConfigParser
    STATS_PARSER = StatsParser

class InputProposeActionsParser(base.InputProposeActionsParser):

    def accept(self, visitor):
        token, alloc, fix = self._parseLine()
        assert token == "InputProposeActions"

        visitor.visitAllocOnly(alloc)
        visitor.visitFixStats(fix)

        visitor.visitEnd()

class ActionType(object):
    (LOWEST_ACTION_UNUSED,
     VMOTION_ACTION,
     STORAGE_VMOTION_ACTION,
     CHANGE_RES_SETTINGS_ACTION,
     POWER_ACTION_DRM,
     INIT_PLACE_ACTION,
     DIAG_INC_LIMIT,
     DIAG_INC_SIZE,
     DIAG_INC_SHARES,
     DIAG_INC_RESERVATION,
     DIAG_DEC_OTHERS_RESERVATION,
     DIAG_INC_CLUSTER_CAPACITY,
     DIAG_DEC_MIG_THRESH,
     HOST_MAINT_ACTION,
     STORAGE_PLACEMENT,
     HIGHEST_ACTION_UNUSED
     ) = range(16)

class ActionBaseParser(base.ActionBaseParser):

    GRAPH_NODE_PARSER = GraphNodeParser

    def accept(self, visitor):
        token, typ, state, auto = self._parseLine()
        assert token == "DrmActionBase"

        visitor.visitType(typ)
        visitor.visitState(state)
        visitor.visitIsAuto(auto)

        v = visitor.visitGraphNode()
        self.GRAPH_NODE_PARSER(self._fp).accept(v)

class ChangeResSettingsParser(base.ChangeResSettingsParser):

    ACTION_BASE_PARSER = ActionBaseParser

class PowerActionDrmParser(TabBasedParser):

    ACTION_BASE_PARSER = ActionBaseParser

    def accept(self, visitor):
        token, on, id, path = self._parseLine()
        assert token == "DrmActionPower"

        visitor.visitAction(id, int(on) != 0, path)

        v = visitor.visitActionBase()
        self.ACTION_BASE_PARSER(self._fp).accept(v)

        visitor.visitEnd()

class VmotionActionParser(TabBasedParser):

    def accept(self, visitor):
        token, _vmId, _srcId, _dstId, \
        _vmPath, _srcHostPath, _dstHostPath = self._parseLine()
        assert token == "DrmActionVMotion"
        visitor.visitVmotionAction(_vmId, _srcId, _dstId,
                                   _vmPath, _srcHostPath, _dstHostPath)

class StorageVmotionActionParser(TabBasedParser):

    ACTION_BASE_PARSER = ActionBaseParser
    VMOTION_ACTION_PARSER = VmotionActionParser

    def accept(self, visitor):
        token, vmId, ndiskIds = self._parseLine()
        assert token == "DrmActionStorageVMotion"

        vals = self._parseLine()
        if ndiskIds > 0:
            vals = map(lambda v: int(v), vals)
            print vals

        self.VMOTION_ACTION_PARSER(self._fp).accept(visitor)

        v = visitor.visitActionBase()
        self.ACTION_BASE_PARSER(self._fp).accept(v)

        visitor.visitEnd()


class ActionParser(base.ActionParser):

    ACTION_TYPE = ActionType
    CHANGE_RES_SETTINGS_PARSER = ChangeResSettingsParser
    POWER_ACTION_DRM_PARSER = PowerActionDrmParser
    STORAGE_VMOTION_ACTION_PARSER = StorageVmotionActionParser

    def _acceptPowerAction(self, visitor):
        v = visitor.visitPowerActionDrm()
        self.POWER_ACTION_DRM_PARSER(self._fp).accept(v)

    def _acceptStorageVmotionAction(self, visitor):
        v = visitor.visitStorageVmotionAction()
        self.STORAGE_VMOTION_ACTION_PARSER(self._fp).accept(v)

    def _dispatch(self, id, visitor):
        if id == self.ACTION_TYPE.POWER_ACTION_DRM:
            self._acceptPowerAction(visitor)
        elif id == self.ACTION_TYPE.STORAGE_VMOTION_ACTION:
            self._acceptStorageVmotionAction(visitor)
        else:
            base.ActionParser._dispatch(self, id, visitor)

class RecommendationParser(base.RecommendationParser):

    ACTION_PARSER = ActionParser
    GRAPH_NODE_PARSER = GraphNodeParser

    def _acceptGraph(self, visitor):
        v = visitor.visitGraphNode()
        self.GRAPH_NODE_PARSER(self._fp).accept(v)

    def accept(self, visitor):
        self._acceptHeader(visitor)
        self._acceptReason(visitor)
        self._acceptActions(visitor)
        self._acceptGraph(visitor)
        visitor.visitEnd()

class ResultsProposeActionsParser(base.ResultsProposeActionsParser):

    RECOMMENDATION_PARSER = RecommendationParser

class PendingActionsParser(base.TabBasedParser):
    RECOMMENDATION_PARSER = RecommendationParser

    def accept(self, visitor):
        token, number = self._parseLine()
        assert token == "PendingActions"

        for _ in range(int(number)):
            self.RECOMMENDATION_PARSER(self._fp).accept(visitor)

        visitor.visitEnd()

class SpiParser(base.SpiParser):

    PENDING_ACTIONS_PARSER = PendingActionsParser

    def accept(self, visitor):
        token, poll, time, clock, wrap, quickstats = self._parseLine(6)
        assert token == "DrmSpi"

        visitor.visitPollInterval(poll)
        visitor.visitCurrentTime(time)
        visitor.visitLastClock(clock)
        visitor.visitNumWrapArounds(wrap)
        visitor.visitQuickStatsLive(quickstats)

        v = visitor.visitPendingActions()
        self.PENDING_ACTIONS_PARSER(self._fp).accept(v)

        visitor.visitEnd()


class ProposeActionsParser(base.ProposeActionsParser):

    INPUT_PROPOSE_ACTIONS_PARSER = InputProposeActionsParser
    RESULTS_PROPOSE_ACTIONS_PARSER = ResultsProposeActionsParser

class DumpParser(base.DumpParser):

    REBALANCE_MODULE_PARSER = RebalanceModuleParser
    PROPOSE_ACTIONS_PARSER = ProposeActionsParser

    def __init__(self, *args, **kwargs):
        base.DumpParser.__init__(self, *args, **kwargs)

    def accept(self, visitor):
        (typeid,) = self._parseLine()
        if typeid.endswith("RebalanceModule"):
            self._acceptRebalanceModule(visitor)
        else:
            raise base.DumpSectionException(typeid)
