from v103 import *
import v103 as base

class VmParser(base.VmParser):

    NODE_PARSER = NodeParser
    ALLOC_PARSER = AllocParser

    def _acceptSections(self, visitor):
        v = visitor.visitAlloc()
        self.ALLOC_PARSER(self._fp).accept(v)

        token, number = self._parseLine()
        assert token == "DrmVm::_recentVmotions"
        for _ in range(int(number)):
            raise Exception("Not implemented: item in %s" % (token))

        token, number = self._parseLine()
        assert token == "DrmVm::_predictedOverheadMap"
        for _ in range(int(number)):
            x, y = self._parseLine()
            visitor.visitPredictedOverhead(x, y)

        token, number = self._parseLine()
        assert token == "DrmVm::_networks"
        for _ in range(int(number)):
            token, type, nports, name, device = self._parseLine()
            visitor.visitNetwork(type, nports, name, device)

        token, number = self._parseLine()
        assert token == "DrmVm::_ftInfo"
        for _ in range(int(number)):
            raise Exception("Not implemented: item in %s" % (token))

        token, number = self._parseLine()
        assert token == "DrmVm::_disks" # to handle the new DrmDisk structures
        for _ in range(int(number)):
            list = self._parseLine()
            token = list[0]
            options = list[1:]
            visitor.visitDisk(token, options)

            (token, val) = self._parseLine()
            assert token == "DrmDisk::_affineDisks"

            # TODO: have to parse affineDisks, if val > 0
            # CPP code needs to be cleaned up to make
            # the new lines consistent

            (token, val) = self._parseLine()
            assert token == "DrmDisk::_incompatibleDisks"

            (token, val) = self._parseLine()
            assert token == "DrmDisk::_requiredFeatures"

            (token, val) = self._parseLine()
            assert token == "DrmDisk::_moveOption"

            (token, val) = self._parseLine()
            assert token == "DrmDisk::_backing"

            if val == '1':
                token, sizeMB, contentId = self._parseLine()
                assert token == "DrmDiskBacking"

                filePath = self._parseLine()

        v = visitor.visitNode()
        self.NODE_PARSER(self._fp).accept(v)

class ConfigVmsParser(base.ConfigVmsParser):

    VM_PARSER = VmParser

class ConfigParser(base.ConfigParser):

    CONFIG_HOSTS_PARSER = ConfigHostsParser
    CONFIG_VMS_PARSER = ConfigVmsParser
    CONFIG_GROUPS_PARSER = ConfigGroupsParser

    CONFIG_DATASTORES_PARSER = ConfigDatastoresParser

    def _acceptSections(self, visitor):
        base.ConfigParser._acceptSections(self, visitor)

class RebalanceModuleParser(base.RebalanceModuleParser):

    OPTIONS_PARSER = OptionsParser
    SPI_PARSER = SpiParser
    CONFIG_PARSER = ConfigParser
    STATS_PARSER = StatsParser

class DumpParser(base.DumpParser):

    REBALANCE_MODULE_PARSER = RebalanceModuleParser
    PROPOSE_ACTIONS_PARSER = ProposeActionsParser
    GRAPH_NODE_PARSER = GraphNodeParser

    def __init__(self, *args, **kwargs):
        base.DumpParser.__init__(self, *args, **kwargs)
        self._sioMapVm = kwargs.get('sioMapVm', False)

    def _parseInputMapVm(self, visitor):
        """
        Parses InputMapVm, which can be in two forms

        InputMapVm 0 <recParents> <n vms>
        <0 ... n - 1 vm ids>

        InputMap 1 <recParents>
        <DrmInitPlaceGroupList -> dump()>
        """
        vals = self._parseLine()
        assert vals[0] == 'InputMapVm'
        if len(vals) == 4:
            assert int(vals[1]) == 0
            recParents = int(vals[2])
            nvms = int(vals[3])
            vmids = self._parseLine()
            if(nvms > 0):
                vmids = map(lambda v: int(v), vmids)
                visitor.visitInputMapVm(recParents, vmids)
        elif len(vals) == 3:
            assert int(vals[1]) == 1
            recParents = int(vals[2])
            self.GRAPH_NODE_PARSER(self._fp).accept(visitor)

    def accept(self, visitor):
        (typeid,) = self._parseLine()
        if typeid.endswith("StorageIOModule"):      # Parsing S-DRS dump file
            self._acceptRebalanceModule(visitor)
            if self._sioMapVm:
                self._parseInputMapVm(visitor)
        elif typeid.endswith("RebalanceModule"):    # C-DRS Rebalance Module
            self._acceptRebalanceModule(visitor)
        else:
            raise base.DumpSectionException(typeid)
