from _helpers import BaseVisitor

class _OptionsVisitor(BaseVisitor):

    def visitOption(self, value):
        pass

class StringOptionsVisitor(_OptionsVisitor):
    pass

class IntOptionsVisitor(_OptionsVisitor):
    pass

class EnumOptionsVisitor(_OptionsVisitor):
    pass

class OptionsVisitor(BaseVisitor):

    STRING_OPTIONS_VISITOR = StringOptionsVisitor
    INT_OPTIONS_VISITOR = IntOptionsVisitor
    ENUM_OPTIONS_VISITOR = EnumOptionsVisitor

    def visitStringOptions(self):
        return self.STRING_OPTIONS_VISITOR()

    def visitIntOptions(self):
        return self.INT_OPTIONS_VISITOR()

    def visitEnumOptions(self):
        return self.ENUM_OPTIONS_VISITOR()

class ResourceAllocVisitor(BaseVisitor):

    def visitReservation(self, value):
        pass

    def visitExpandableReservation(self, value):
        pass

    def visitLimit(self, value):
        pass

    def visitOverheadLimit(self, value):
        pass

    def visitShares(self, shares, level):
        pass

class ResourceConfigSpecVisitor(BaseVisitor):

    RESOURCE_ALLOC_VISITOR = ResourceAllocVisitor

    def visitCpuAllocation(self):
        return self.RESOURCE_ALLOC_VISITOR()

    def visitMemAllocation(self):
        return self.RESOURCE_ALLOC_VISITOR()

class AllocVisitor(BaseVisitor):

    RESOURCE_CONFIG_SPEC_VISITOR = ResourceConfigSpecVisitor

    def visitResourceConfigSpec(self):
        return self.RESOURCE_CONFIG_SPEC_VISITOR()

    def visitEmpty(self, empty, id):
        pass

class ObjectVisitor(BaseVisitor):

    def visitId(self, value):
        pass

    def visitUuid(self, value):
        pass

    def visitPath(self, value):
        pass

    def visitName(self, value):
        pass

class NodeVisitor(BaseVisitor):

    ALLOC_VISITOR = AllocVisitor
    OBJECT_VISITOR = ObjectVisitor

    def visitId(self, value):
        pass

    def visitName(self, value):
        pass

    def visitAlloc(self):
        return self.ALLOC_VISITOR()

    def visitObject(self):
        return self.OBJECT_VISITOR()

class HostVisitor(BaseVisitor):

    OBJECT_VISITOR = ObjectVisitor

    def visitInVmotion(self, value):
        pass

    def visitHostState(self, value):
        pass

    def visitNumCpuCores(self, value):
        pass

    def visitNumCpuThreads(self, value):
        pass

    def visitCpuMhz(self, value):
        pass

    def visitBootTime(self, value):
        pass

    def visitShutDownTime(self, value):
        pass

    def visitTimeSinceLastExitStandby(self, value):
        pass

    def visitCpu(self, value):
        pass

    def visitMemory(self, value):
        pass

    def visitAutoLevel(self, value):
        pass

    def visitSupportsFSR(self, value):
        pass

    def visitWakeableNotUsingWol(self, value):
        pass

    def visitVersion(self, value):
        pass

    def visitMaxVms(self, value):
        pass

    def visitMaxVcpus(self, value):
        pass

    def visitBuild(self, value):
        pass

    def visitName(self, value):
        pass

    def visitConnectedDatastore(self, value):
        pass

    def visitCapacity(self, capacity, total, reserved=None):
        pass

    def visitObject(self):
        return self.OBJECT_VISITOR()

    def visitDvs(self, name, value):
        pass

    def visitSwitch(self, name, value):
        pass

    def visitNetworkSwitch(self, network, switch):
        pass

class HostsVisitor(BaseVisitor):

    HOST_VISITOR = HostVisitor

    def visitHost(self):
        return self.HOST_VISITOR()

class VmVisitor(BaseVisitor):

    ALLOC_VISITOR = AllocVisitor
    NODE_VISITOR = NodeVisitor

    def visitAutomationLevel(self, value):
        pass

    def visitDestHostId(self, value):
        pass

    def visitRunningOn(self, value):
        pass

    def visitVmState(self, value):
        pass

    def visitNumVcpus(self, value):
        pass

    def visitMem(self, value):
        pass

    def visitOverhead(self, value):
        pass

    def visitSwappedOverhead(self, value):
        pass

    def visitIsAgentVm(self, value):
        pass

    def visitIsTemplateVm(self, value):
        pass

    def visitDisksAffined(self, value):
        pass

    def visitCompatibleHost(self, value):
        pass

    def visitCompatibleHostNoHardRules(self, value):
        pass

    def visitPreferedHost(self, value):
        pass

    def visitIncompatibleVm(self, value):
        pass

    def visitAffineVm(self, value):
        pass

    def visitAlloc(self):
        return self.ALLOC_VISITOR()

    def visitPredictedOverhead(self, x, y):
        pass

    def visitNetwork(self, type, nports, name, device):
        pass

    # type = DrmDisk || DrmDisk::_YYY
    #options = (diskId, vmId, id, datastoreId, dstDatastoreId, \
    #committedMB, uncommittedMB, \
    #diskSizeMB, diskState, dasClusterId,  isDrmEnabled, \
    #isInAllDisksAffined,  shares,  hiddenFilesSameDs, \
    #hiddenFilesOtherDs,  name)
    def visitDisk(self, type, options):
        pass

    def visitNode(self):
        return self.NODE_VISITOR()

class VmsVisitor(BaseVisitor):

    VM_VISITOR = VmVisitor

    def visitVm(self):
        return self.VM_VISITOR()

class GroupVisitor(BaseVisitor):

    ALLOC_VISITOR = AllocVisitor
    NODE_VISITOR = NodeVisitor

    def visitId(self, id):
        pass

    def visitAlloc(self):
        return self.ALLOC_VISITOR()

    def visitNode(self):
        return self.NODE_VISITOR()

class GroupsVisitor(BaseVisitor):

    GROUP_VISITOR = GroupVisitor

    def visitGroup(self):
        return self.GROUP_VISITOR()

class HostsWithStatsVisitor(BaseVisitor):

    def visitHost(self, value):
        pass

class VmsWithStatsVisitor(BaseVisitor):

    def visitVm(self, value):
        pass

class VpxPath2IdHostsVmsVisitor(BaseVisitor):

    def visitPathId(self, id, path):
        pass

class DatastoresVisitor(ObjectVisitor):
    OBJECT_VISITOR = ObjectVisitor

    def visitDatastoreOptions(self, options):
        pass

    def visitVmIds(self):
        pass

    def visitCorrelatedDataStores(self):
        pass

    def visitConnectedHosts(self):
        pass

    def visitHaMasters(self):
        pass

    def  visitSupportedFeatures(self):
        pass

    def visitObject(self):
        return self.OBJECT_VISITOR()

class ConfigVisitor(BaseVisitor):

    HOSTS_VISITOR = HostsVisitor
    VMS_VISITOR = VmsVisitor
    GROUPS_VISITOR = GroupsVisitor
    HOSTS_WITH_STATS_VISITOR = HostsWithStatsVisitor
    VMS_WITH_STATS_VISITOR = VmsWithStatsVisitor
    VPX_PATH_2_ID_HOSTS_VMS_VISITOR = VpxPath2IdHostsVmsVisitor
    DATASTORES_VISITOR = DatastoresVisitor

    def visitStatus(self, value):
        pass

    def visitVmotionRate(self, value):
        pass

    def visitHostPowerActionRate(self, value):
        pass

    def visitRoot(self, value):
        pass

    def visitTimestamp(self, value):
        pass

    def visitHosts(self):
        return self.HOSTS_VISITOR()

    def visitVms(self):
        return self.VMS_VISITOR()

    def visitGroups(self):
        return self.GROUPS_VISITOR()

    def visitHostsWithStats(self):
        return self.HOSTS_WITH_STATS_VISITOR()

    def visitVmsWithStats(self):
        return self.VMS_WITH_STATS_VISITOR()

    def visitVpxPath2IdHostsVms(self):
        return self.VPX_PATH_2_ID_HOSTS_VMS_VISITOR()

    def visitDatastores(self):
        return self.DATASTORES_VISITOR()

class RecentStatsVisitor(BaseVisitor):

    def visitMaxSize(self, value):
        pass

    def visitTimeoutSec(self, value):
        pass

    def visitTimeout(self, value):
        pass

    def visitFakeStatTime(self, value):
        pass

    def visitType(self, value):
        pass

    def visitTimestamp(self, value):
        pass

    def visitValue(self, value):
        pass

class ObjectStatsVisitor(BaseVisitor):

    RECENT_STATS_VISITOR = RecentStatsVisitor

    def visitId(self, value):
        pass

    def visitSize(self, value):
        pass

    def visitStatsAvailable(self, value):
        pass

    def visitTimeoutSec(self, value):
        pass

    def visitRecentStats(self, id):
        return self.RECENT_STATS_VISITOR()

class HostStatsVisitor(BaseVisitor):

    OBJECT_STATS_VISITOR = ObjectStatsVisitor

    def visitId(self, value):
        pass

    def visitObjectStats(self):
        return self.OBJECT_STATS_VISITOR()

class VmStatsVisitor(BaseVisitor):

    OBJECT_STATS_VISITOR = ObjectStatsVisitor

    def visitId(self, value):
        pass

    def visitObjectStats(self):
        return self.OBJECT_STATS_VISITOR()

class DatastoreStatsVisitor(BaseVisitor):

    OBJECT_STATS_VISITOR = ObjectStatsVisitor

    def visitObjectStats(self):
        return self.OBJECT_STATS_VISITOR()

    def visitDSSamples(self, histid, samples):
        pass


class VmdkStatsVisitor(BaseVisitor):

    OBJECT_STATS_VISITOR = ObjectStatsVisitor

    def visitObjectStats(self):
        return self.OBJECT_STATS_VISITOR()

    def visitVmdkSamples(self, histid, samples):
        pass


class StatsVisitor(BaseVisitor):

    HOST_STATS_VISITOR = HostStatsVisitor
    VM_STATS_VISITOR = VmStatsVisitor
    DATASTORE_STATS_VISITOR = DatastoreStatsVisitor
    VMDK_STATS_VISITOR = VmdkStatsVisitor
    RECENT_STATS_VISITOR = RecentStatsVisitor

    def visitTimeout(self, value):
        pass

    def visitMaxhistSize(self, value):
        pass

    def visitStatId(self, value):
        pass

    def visitHostStats(self):
        return self.HOST_STATS_VISITOR()

    def visitVmStats(self):
        return self.VM_STATS_VISITOR()

    def visitDatastoreStats(self):
        return self.DATASTORE_STATS_VISITOR()

    def visitVmdkStats(self):
        return self.VMDK_STATS_VISITOR()

    def visitRecentStats(self):
        return self.RECENT_STATS_VISITOR()

class InputProposeActionsVisitor(BaseVisitor):

    def visitAllocOnly(self, value):
        pass

    def visitFixStats(self, value):
        pass

class GraphNodeVisitor(BaseVisitor):

    def visitPos(self, value):
        pass

    def visitPrereqPos(self, value):
        pass

class ActionBaseVisitor(BaseVisitor):

    GRAPH_NODE_VISITOR = GraphNodeVisitor

    def visitType(self, value):
        pass

    def visitState(self, value):
        pass

    def visitIsAuto(self, value):
        pass

    def visitGraphNode(self):
        return self.GRAPH_NODE_VISITOR()

class ChangeResSettingsVisitor(BaseVisitor):

    RESOURCE_CONFIG_SPEC_VISITOR = ResourceConfigSpecVisitor
    ACTION_BASE_VISITOR = ActionBaseVisitor

    def visitIsVm(self, value):
        pass

    def visitTarget(self, value):
        pass

    def visitHost(self, value):
        pass

    def visitResSettingsType(self, value):
        pass

    def visitEntityPath(self, value):
        pass

    def visitHostPath(self, value):
        pass

    def visitResourceConfigSpec(self):
        return self.RESOURCE_CONFIG_SPEC_VISITOR()

    def visitActionBase(self):
        return self.ACTION_BASE_VISITOR()

class PowerActionDrmVisitor(BaseVisitor):

    ACTION_BASE_VISITOR = ActionBaseVisitor

    def visitAction(self, host_id, turn_on=False, path=""):
        pass

    def visitActionBase(self):
        return self.ACTION_BASE_VISITOR()

class StorageVmotionActionVisitor(PowerActionDrmVisitor):
    pass

class VmotionActionVisitor(PowerActionDrmVisitor):
    pass

class ActionVisitor(BaseVisitor):

    CHANGE_RES_SETTINGS_VISITOR = ChangeResSettingsVisitor
    POWER_ACTION_DRM_VISITOR = PowerActionDrmVisitor
    STORAGE_VMOTION_ACTION_VISITOR = StorageVmotionActionVisitor
    VMOTION_ACTION_VISITOR = VmotionActionVisitor

    def visitChangeResSettings(self):
        return self.CHANGE_RES_SETTINGS_VISITOR()

    def visitPowerActionDrm(self):
        return self.POWER_ACTION_DRM_VISITOR()

    def visitStorageVmotionAction(self):
        return self.STORAGE_VMOTION_ACTION_VISITOR()

    def visitVmotionAction(self, vmId, srcId, dstId,
                                 vmPath, srcHostPath, dstHostPath):
        return self.VMOTION_ACTION_VISITOR()

class RecommendationVisitor(BaseVisitor):

    ACTION_VISITOR = ActionVisitor
    GRAPH_NODE_VISITOR = GraphNodeVisitor

    def visitRating(self, value):
        pass

    def visitId(self, value):
        pass

    def visitState(self, value):
        pass

    def visitReasonCode(self, value):
        pass

    def visitReasonText(self, value):
        pass

    def visitPrereq(self, value):
        pass

    def visitAction(self):
        return self.ACTION_VISITOR()

    def visitGraphNode(self):
        return self.GRAPH_NODE_VISITOR()

class ResultsProposeActionsVisitor(BaseVisitor):

    RECOMMENDATION_VISITOR = RecommendationVisitor

    def visitStatus(self, value):
        pass

    def visitRecommendation(self):
        return self.RECOMMENDATION_VISITOR()

class PendingActionsVisitor(BaseVisitor):
    ACTION_VISITOR = ActionVisitor
    ACTION_BASE_VISITOR = ActionBaseVisitor
    GRAPH_NODE_VISITOR = GraphNodeVisitor

    def visitRating(self, rating):
        pass

    def visitId(self, id):
        pass

    def visitState(self, state):
        pass

    def visitReasonCode(self, reasonCode):
        pass

    def visitReasonText(self, reasonText):
        pass

    def visitAction(self):
        return self.ACTION_VISITOR()

    def visitActionBase(self):
        return self.ACTION_BASE_VISITOR()

    def visitGraphNode(self):
        return self.GRAPH_NODE_VISITOR()


class SpiVisitor(BaseVisitor):

    PENDING_ACTIONS_VISITOR = PendingActionsVisitor

    def visitPollInterval(self, value):
        pass

    def visitCurrentTime(self, value):
        pass

    def visitLastClock(self, value):
        pass

    def visitNumWrapArounds(self, value):
        pass

    def visitQuickStatsLive(self, value):
        pass

    def visitPendingActions(self):
        return self.PENDING_ACTIONS_VISITOR()



class ProposeActionsVisitor(BaseVisitor):

    INPUT_PROPOSE_ACTIONS_VISITOR = InputProposeActionsVisitor
    RESULTS_PROPOSE_ACTIONS_VISITOR = ResultsProposeActionsVisitor

    def visitInputProposeActions(self):
        return self.INPUT_PROPOSE_ACTIONS_VISITOR()

    def visitResultsProposeActions(self):
        return self.RESULTS_PROPOSE_ACTIONS_VISITOR()

class RebalanceModuleVisitor(BaseVisitor):

    OPTIONS_VISITOR = OptionsVisitor
    SPI_VISITOR = SpiVisitor
    CONFIG_VISITOR = ConfigVisitor
    STATS_VISITOR = StatsVisitor

    def visitFairnessTimestamp(self, value):
        pass

    def visitMigrateTimestamp(self, value):
        pass

    def visitOptions(self):
        return self.OPTIONS_VISITOR()

    def visitSpi(self):
        return self.SPI_VISITOR()

    def visitConfig(self):
        return self.CONFIG_VISITOR()

    def visitStats(self):
        return self.STATS_VISITOR()

class DumpVisitor(BaseVisitor):

    REBALANCE_MODULE_VISITOR = RebalanceModuleVisitor
    PROPOSE_ACTIONS_VISITOR = ProposeActionsVisitor

    def visitRebalanceModule(self):
        return self.REBALANCE_MODULE_VISITOR()

    def visitProposeActions(self):
        return self.PROPOSE_ACTIONS_VISITOR()

    def visitInputMapVm(self, recParents, vmids):
        pass

class DumpTool(DumpVisitor):

    def __init__(self, *args, **kwargs):
        pass

    def visitVersion(self, value):
        pass
