import matplotlib
from wxPython.wx import *
from wxPython.grid import *
from matplotlib.backends.backend_wxagg import *
from matplotlib.figure import Figure
from matplotlib.axes import Subplot

import sys, optparse
import drmplot, drmparse 
from drmdata import *

MAX_CHARTS = 8

ids = {}
nextId = 100
def mkid(name):
    if ids.has_key(name):
        return ids[name]
    else:
        global nextId
        ids[name] = nextId
        nextId += 1
        return nextId-1

TYPE_COL = 0
NAME_COL = 1
STATTYPE_COL = 2
VAL1_COL = 3
VAL2_COL = 4
VAL3_COL = 5

def MakeCanvasWx(parent, id, figure):
    return FigureCanvasWxAgg(parent, id, figure)

class ConfigFrame(wxFrame):
    def SetupValEditors(self, r, typ, oldlist):
        typlist = self.valtypes[typ]
        edit = wxGridCellChoiceEditor(choices = typlist)
        numMissing = 3 - len(oldlist)
        for i in range(numMissing):
            oldlist.append("N/A")
        self.grid.SetCellEditor(r, VAL1_COL, edit)
        self.grid.SetCellValue(r, VAL1_COL, oldlist[0])
        self.grid.SetCellEditor(r, VAL2_COL, edit)
        self.grid.SetCellValue(r, VAL2_COL, oldlist[1])
        self.grid.SetCellEditor(r, VAL3_COL, edit)
        self.grid.SetCellValue(r, VAL3_COL, oldlist[2])

    def GetConfigElements(self):
        elements = []
        for i in range(MAX_CHARTS):
            thisrow = []

            # helper to get value in row
            def GRV(c):
                v = self.grid.GetCellValue(i, c)
                if v == "N/A":
                    return None
                else:
                    return v
                
            typ = GRV(TYPE_COL)
            if typ is None:
                continue
            vm = self.parser.GetNode(typ,GRV(NAME_COL))
            vallist = [GRV(VAL1_COL), GRV(VAL2_COL), GRV(VAL3_COL)]
            vallist = filter(lambda x: x is not None, vallist)
            if len(vallist) == 0:
                continue
            pelem = drmplot.PlotElement(vm, GRV(STATTYPE_COL), vallist)
            elements.append(pelem)
            
        return elements

    def GetWantLegend(self):
        return self.legendBox.IsChecked()
        
    def SetupGrid(self, elements):
        self.grid = wxGrid(self, mkid("configgrid"))
        self.grid.CreateGrid(MAX_CHARTS,6)
        self.grid.SetColLabelValue(TYPE_COL, "Type")
        self.grid.SetColLabelValue(NAME_COL, "Name")
        self.grid.SetColSize(NAME_COL, self.grid.GetColSize(NAME_COL) * 2)
        self.grid.SetColLabelValue(STATTYPE_COL, "StatType")
        self.grid.SetColLabelValue(VAL1_COL, "Val1")
        self.grid.SetColLabelValue(VAL2_COL, "Val2")
        self.grid.SetColLabelValue(VAL3_COL, "Val3")

        self.grid.SetRowLabelSize(20)

        # setup several different types of combo boxes for the grid
        self.valtypes = {}
        entityTypes = ["N/A", "group", "host", "vm"]
        for t in entityTypes:
            self.valtypes[t] = ["N/A"]
            self.valtypes[t].extend(self.parser.GetValTypes(t))
        typEdit = wxGridCellChoiceEditor(choices = entityTypes)
        statTypes = self.parser.GetStatTypes()
        statTypeEdit = wxGridCellChoiceEditor(choices = statTypes)
        
        # TODO : switch to a damn table model!!!!
        
        for r in range(MAX_CHARTS):
            if r < len(elements):
                thiselem = elements[r]
                if isinstance(thiselem.node, drmparse.Host):
                    typ = "host"
                elif isinstance(thiselem.node, drmparse.VM):
                    typ = "vm"
                elif isinstance(thiselem.node, drmparse.Hose):
                    typ = "host"
                else:
                    assert False, "unknown element node"
                name = thiselem.node.name
                stattype = thiselem.stattypes[0] # TODO support multiple types
                vtypes = thiselem.valtypes
            else:
                typ = "N/A"
                name = "N/A"
                stattype = "ActAvg1Min"
                vtypes = []
            namelist = self.parser.GetNodeNames(typ)
            nameEdit = wxGridCellChoiceEditor(choices = namelist)
                    
            self.grid.SetCellEditor(r, TYPE_COL, typEdit)
            self.grid.SetCellEditor(r, NAME_COL, nameEdit)
            self.grid.SetCellEditor(r, STATTYPE_COL, statTypeEdit)

            self.grid.SetCellValue(r, TYPE_COL, typ)
            self.grid.SetCellValue(r, NAME_COL, name)
            self.grid.SetCellValue(r, STATTYPE_COL, stattype)
            
            self.SetupValEditors(r, typ, vtypes)

    def OkEvent(self, evt):
        print "accepted reconfigure"
        self.grid.SaveEditControlValue()
        self.parent.SetNewConfig(self)
        self.Hide()
        

    def CancelEvent(self, evt):
        print "cancelled reconfigure"
        self.Hide()

    def CellChange(self, evt):
        r, c = evt.GetRow(), evt.GetCol()
        print 'cell change at row ', r
        if c == TYPE_COL:
            # update the names list to show names for
            # the selected node type
            typ = self.grid.GetCellValue(r, TYPE_COL)
            print 'type col changed, need to update to ', typ
            oldname = self.grid.GetCellValue(r, NAME_COL)
            namelist = ["N/A"] + self.parser.GetNodeNames(typ)
            nameEdit = wxGridCellChoiceEditor(choices = namelist)
            self.grid.SetCellEditor(r, NAME_COL, nameEdit)
            if oldname not in namelist:
                self.grid.SetCellValue(r, NAME_COL, "N/A")
            self.SetupValEditors(r, typ, [])

    def __init__(self, parent, id, parser, elements):
        wxFrame.__init__(self, parent, id, "Configure Display")

        self.parser = parser
        self.parent = parent
        
        self.SetupGrid(elements)
        
        self.SetBackgroundColour(wxLIGHT_GREY)
        sizer = wxBoxSizer(wxVERTICAL)
        sizer.Add(self.grid, 1, wxTOP|wxBOTTOM|wxLEFT|wxRIGHT|wxGROW)

        hsizer = wxBoxSizer(wxHORIZONTAL)
        hsizer.Add((20,0))

        self.legendBox = wxCheckBox(self, -1, "Display Legend?")
        self.SetBackgroundColour(self.legendBox.GetBackgroundColour())
        hsizer.Add(self.legendBox, 0, wxALIGN_CENTER_VERTICAL)
        hsizer.Add((150,0))
        hsizer.Add(wxButton(self, mkid("okbutton"), label="Ok"), 0)
        hsizer.Add((10,0))
        hsizer.Add(wxButton(self, mkid("cancelbutton"), label="Cancel"), 0)

        EVT_BUTTON(self, mkid("okbutton"), self.OkEvent)
        EVT_BUTTON(self, mkid("cancelbutton"), self.CancelEvent)    
        EVT_GRID_CELL_CHANGE(self.grid, self.CellChange)
        sizer.Add((0, 5))
        sizer.Add(hsizer)
        sizer.Add((0, 5))
        
        self.SetSizer(sizer)
        self.SetSize( (600, 300) )
        
class DetailGrid(wxGrid):
    def LabelClick(self, evt):
        c = evt.GetCol()
        self.tdata.SortData(c)
        self.ForceRefresh()
        
    def LoadData(self, table):
        self.tdata.table = table
        self.tdata.Refresh()

    def __init__(self, parent, id, tbl):
        wxGrid.__init__(self, parent, id)
        self.EnableEditing(False)
        self.tdata = TableDataWx(tbl)
        self.tdata.SortData(0)
        self.SetTable(self.tdata)
        self.SetColSize(0, self.GetColSize(0) * 2)
        self.SetRowLabelSize(0)
        
        EVT_GRID_LABEL_LEFT_CLICK(self, self.LabelClick)
        
class TableDataWx(wxPyGridTableBase):
    def GetValue(self, row, col):
        return self.table.GetValue(row, col)
    def GetNumberRows(self):
        return self.table.GetNumberRows()
    def GetNumberCols(self):
        return self.table.GetNumberCols()
    def SortData(self, keynum):
        return self.table.SortData(keynum)
    def GetColLabelValue(self, col):
        return self.table.GetColLabelValue(col)
    def ToHtml(self):
        return self.table.ToHtml()
    def Refresh(self):
        """Update all displayed values"""
        view = self.GetView()
        msg = wxGridTableMessage(self, wxGRIDTABLE_REQUEST_VIEW_GET_VALUES)
        view.ProcessTableMessage(msg) 

    def __init__(self, table):
        wxPyGridTableBase.__init__(self)
        self.table = table
        
class PlotFrame(wxFrame):
    def on_click(self, event):
        self.getPlotMgr().HandleClick(event)
        self.canvas.draw()
        self.passnum = self.getPlotMgr().plotter.GetSelectedPassnum()+1

        self.datamgr.SetupTables(self.passnum)
        
        for gkey in self.datamgr.tables.keys():
            mytable = self.datamgr.tables[gkey]
            self.grids[gkey].LoadData(mytable)
            
        self.passLabel.SetLabel("Selected Pass: " + str(self.passnum))
        self.passLabel.Refresh()

    def SetupCanvas(self):
        self.getPlotMgr().doPlot(self.wantLegend)
        self.SetBackgroundColour(wxLIGHT_GREY)
        wxcanvas = MakeCanvasWx(self, -1, self.getPlotMgr().plotter)
        self.getPlotMgr().setCanvas(wxcanvas)
        self.canvas = wxcanvas
        self.canvas.mpl_connect('button_press_event', self.on_click)
        self.vertSiz.Prepend(self.canvas, 4, wxLEFT|wxTOP|wxGROW)
        
    def RefreshEvent(self, evt):
        self.vertSiz.Remove(self.canvas)
        self.canvas.Destroy()
        self.SetupCanvas()
        self.vertSiz.Layout()
        self.Fit()

    def SetNewConfig(self, cfg):
        elements = []
        elems = cfg.GetConfigElements()
        for e in elems:
            print e.node.name
        if len(elems) == 0:
            print "No valid elements configured, aborting reconfigure"
            return
        self.wantLegend = cfg.GetWantLegend()
        self.getPlotMgr().setPlotElements(elems)
        self.RefreshEvent(None)

    def ConfigEvent(self, evt):
        cfg = ConfigFrame(self, mkid("cframe"), self.getParser(), self.getPlotMgr().plotElements)
        cfg.Show()

    def getParser(self):
        return self.datamgr.parser

    def getPlotMgr(self):
        return self.datamgr.plotmgr

    def SaveEvent(self, evt):
        dlg = wxFileDialog(self,
                           message='Save image as...',
                           defaultDir=os.getcwd())
        if dlg.ShowModal() == wxID_OK:
            outp = dlg.GetPath()
            self.datamgr.doSave(outp)
 
    def __init__(self, inputdir):
        wxFrame.__init__(self, None, -1, "DRM effectiveness plotting")

        self.inputdir = inputdir
        
        # load the config files and start off my plotting a few
        # miscellaneous stats from four vms
        self.wantLegend = True
        
        self.datamgr = DataMgr(inputdir)
         
        self.vertSiz = wxBoxSizer(wxVERTICAL)
        self.canvas = None
        self.passnum = 0
        self.SetupCanvas()

        hsiz = wxBoxSizer(wxHORIZONTAL)
        hsiz.Add((20,0))
        self.passLabel = wxStaticText(self, -1, "No Pass Num selected")
        hsiz.Add(self.passLabel)
        hsiz.Add((200,10))

        hsiz.Add(wxButton(self, mkid("configbutton"), label="Configure"), 0,
                          wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL)
        hsiz.Add((50,10))
        hsiz.Add(wxButton(self, mkid("savebutton"), label="Save"), 0,
                 wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL)
        hsiz.Add((50,10))
        hsiz.Add(wxButton(self, mkid("refreshbutton"), label="Refresh"), 0,
                 wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL)
        
        EVT_BUTTON(self, mkid("savebutton"), self.SaveEvent)
        EVT_BUTTON(self, mkid("refreshbutton"), self.RefreshEvent)
        EVT_BUTTON(self, mkid("configbutton"), self.ConfigEvent)

        self.notebook = wxNotebook(self, mkid("notebook"))

        self.notebook.SetSize((1000,400))

        self.grids = {}
        for tkey in self.datamgr.tables.keys():
            mytable = self.datamgr.tables[tkey]
            dgrid = DetailGrid(self.notebook, mkid(tkey), mytable)
            self.grids[tkey] = dgrid
            self.notebook.AddPage(dgrid, tkey)

        self.vertSiz.Add(self.notebook, 1, wxLEFT|wxGROW|wxBOTTOM)
        self.vertSiz.Add((0, 5) )

        self.vertSiz.Add(hsiz)
        self.vertSiz.Add((0, 5))
        self.SetSizer(self.vertSiz)
        self.SetSize( (1000,900) )

if __name__ == "__main__":
    opts = optparse.OptionParser()
    opts.add_option("-o", "--output", dest="outdir", default=None)
    options, args = opts.parse_args()
    
    app = wxPySimpleApp(0)
    inputdir = args[0]

    if options.outdir:
        matplotlib.use('Agg')
        data = DataMgr(inputdir)
        print "outdir = ", options.outdir
        if not os.path.exists(options.outdir):
            if not os.mkdir(options.outdir):
                print "could not create directory ", options.outdir
                sys.exit(-1)
        data.doSave(os.path.join(options.outdir, "index.html"))
        sys.exit(0)
    else:
        matplotlib.use('WXAgg')
        frame = PlotFrame(inputdir)
        frame.Show()
        app.MainLoop()


