#exclude default_hosts
NumUnreservedSlots = 0

# Round 1: For this cluster, hosts are required to have 1 agent vm deployed
# to handle client (i.e., non-agent) vms.  DrmBigCpu1 has an agent vm
# (PinnedBigCpu1Vm1), but DrmBigCpu2 does not; hence, client vm ThirdWorkload3
# can only be placed on DrmBigCpu1.
#
# Round 2: When host DrmBigCpu1 is requested to enter maintenance mode,
# DRS cannot evacuate its client vm(s) to DrmBigCpu2 due to its having
# insufficient agent vms deployed; it generates a fault for the client vm.
# Note that it does not generate a fault for the agent vm, because it
# knows that agent vms do not have to be evacuated.
#
# Round 3: Host DrmBigCpu2 now has the required agent vm deployed, so
# DRS can and does now recommend evacuation of DrmBigCpu1's client vm.
# It does not recommend evacuation of VM PinnedBigCpu1Vm1 because it is an
# agent VM (i.e., a pinned per-host service VM).
#
# Round 4: EAM automatically powers-off all agent VMs after all client (i.e.,
# non-agent) VMs are VMotioned off a host entering maint (or standby) mode; to
# model this EAM behavior, set agent VM (PinnedBigCpu1Vm1) state to powered-off.
# Also, turn off the agent status of this VM, since simulator always keeps
# around the powered-off VMs and since it is a agent, DRS will try to honor the
# reservation of this VM even when the host is down. In real VC case, this VM
# will never make it to the snapshot since the host is already down.
#
# Round 5: DrmBigCpu1 is requested to exit maint mode.
#
# Round 6: DrmBigCpu1 completes exit maint mode.  EAM automatically powers-on
# all agent VMs after a host exits maint or standby mode; to model this EAM
# behavior, set agent VM state to powered-on.  DPM is disabled on DrmBigCpu2;
# DPM is able to recommend powering off DrmBigCpu1, understanding that its
# agent vm does not block that recommendation or need to be evacuated.
#
# Round 7: DrmBigCpu1 is in standby, with a representation of its agent
# vm's resource reservations to ensure any request to power-on a vm on
# that host will take those reserved resources into account.
#
# Round 8: Attempt to power-on vm on DrmBigCpu1 that won't fit if agent vm's
# reservations are taken into account (but would if they weren't); fault
# reported.
#
# Round 9: Request to power-on vm on DrmBigCpu1 that will fit.  Works

numrounds=9

host DrmBigCpu 2
vm PinnedBigCpu1Vm		1	   user DrmBigCpu1
vm PinnedBigCpu2Vm		1	   user DrmBigCpu2
vm ThirdWorkload		1	   user delayplace 
vm PinnedBigCpu1Vm		1	   user delayplace
vm PinnedBigCpu1Vma		1	   user delayplace

event:setadvoption NumAgentVmsRequired 1 1
event:agentVmTag PinnedBigCpu1Vm1 1 on
event:place ThirdWorkload3 1 randhost

event:hostStateChange DrmBigCpu1 2 DRM_HOST_GOING_DOWN

event:agentVmTag PinnedBigCpu2Vm2 3 on

event:vmStateChange PinnedBigCpu1Vm1 4 DRM_VM_POWERED_OFF
event:agentVmTag PinnedBigCpu1Vm1 4 off

event:hostStateChange DrmBigCpu1 5 DRM_HOST_OK

event:vmStateChange PinnedBigCpu1Vm1 6 DRM_VM_POWERED_ON
event:agentVmTag PinnedBigCpu1Vm1 6 on
event:hostAutoLevelChange DrmBigCpu2 6 DRM_DISABLED

event:vmStateChange PinnedBigCpu1Vm1 7 DRM_VM_POWERED_OFF

event:place PinnedBigCpu1Vm4 8 DrmBigCpu1

event:place PinnedBigCpu1Vma5 9 DrmBigCpu1

