# At the start of pass 1, the 2 real VMs are on DrmSmallCpu3 which
# the user has requested to enter standby, so they need to be evacuated.
# They are evacuated to the 2 powered-on hosts, DrmSmallCpu1 & Cpu2.
# Both DrmSmallCpu4 (initially) & Cpu3 (after evac) are mapped to standby,
# but are not needed/recommended for power-on.

# At the start of pass 2, the 2 real VMs are on DrmSmallCpu1/Cpu2, the 2
# spare VMs are on DrmSmallCpu2, DrmSmallCpu3 & DrmSmallCpu4 are in
# standby, min-powered-on-cpu-capacity is increased from 4000 to 6000
# and DrmSmallCpu1 is been requested to enter maintenance mode.
# The 4 VMs (2 real and 2 spare) comprise 4000 MHz of cpu capacity,
# which requires 2 of the hosts to be powered on; DRS is able to
# evacuate DrmSmallCpu1 without violating min-powered-on-capacity by
# by scheduling the power-on of DrmSmallCpu3 to directly handle the
# evacuation of DrmSmallCpu1 and the power-on of DrmSmallCpu4 to
# supplement that capacity to reach min and hence allow the evac moves.

# Host DrmSmallCpu1 exits maintenance mode at the end of pass 3, &
# min-powered-on-capacity is increased to 8000 MHz in that pass.

# At the start of pass 4, the 2 real VMs are on DrmSmallCpu3 & DrmSmallCpu4,
# the 2 spare VMs are on DrmSmallCpu1 & DrmSmallCpu2, & DrmSmallCpu3
# is requested to enter maintenance mode.  This time DRS cannot evacuate
# DrmSmallCpu3 without violating min-powered-on-capacity & should not
# recommend any moves for entering maintenance mode.  This engenders
# InsufficientFailoverResourcesFault

# At pass 5 (like pass 4), DRS cannot evacuate DrmSmallCpu3 w/o violating
# min-powered-on-capacity, but this time we simulate that min-cap was not
# set by HA.  This engenders InsufficientStandbyCpuResource.

# At pass 6, all 4 hosts have outstanding requests to enter maint mode,
# leaving no active hosts in the cluster.  Again, DRS should not recommend
# any evacuation moves.  This engenders NoActiveHostInCluster fault.

#exclude default_hosts

numrounds = 6
MinPoweredOnCpuCapacity = 4000
RequireMinCapOnStrictHaAdmit = 1
PowerStarThreshold = 4

host DrmSmallCpu 4
vm FullBusyLargeMin 2 user DrmSmallCpu3

event:hostStateChange DrmSmallCpu3 1 DRM_HOST_ENTER_STANDBY
event:hostStateChange DrmSmallCpu4 1 DRM_HOST_ENTER_STANDBY

event:hostStateChange DrmSmallCpu1 2 DRM_HOST_GOING_DOWN
event:setadvoption MinPoweredOnCpuCapacity 2 6000

event:hostStateChange DrmSmallCpu1 3 DRM_HOST_OK
event:setadvoption MinPoweredOnCpuCapacity 3 8000

event:hostStateChange DrmSmallCpu3 4 DRM_HOST_GOING_DOWN

event:setadvoption RequireMinCapOnStrictHaAdmit 5 0
event:setadvoption NumUnreservedSlots 5 0

event:hostStateChange DrmSmallCpu1 6 DRM_HOST_GOING_DOWN
event:hostStateChange DrmSmallCpu2 6 DRM_HOST_GOING_DOWN
event:hostStateChange DrmSmallCpu4 6 DRM_HOST_GOING_DOWN

