#
# script to check pass criteria of the clone test
#

#
# Extract number of simulation rounds
#
numRounds = 0
roundEx = RegxExtract("dummydrm finished (\d+) rounds")
if len(roundEx) == 1:
    numRounds = int(roundEx[0])
else:
    print 'Unable to determine test rounds'
    fail = 1

round = 1
backEx = "IOLB-Init\]: backing:(\d+) size:(\d+) refs:(\d+) depth:(\d+) "
backEx += "file:EMPTY_STRING datastore:(\d+) disk:(\d+)"
dsEx = "IOLB-Init\]: datastore (\d+)(.+)committed disk (\d+)"
diskEx = "IOLB-Init\]: disk (\d+) \(DrmId (\d+)\): committedMB (\d+)(.+) "
diskEx += "datastore (\d+)"
while (round < numRounds) and (fail == 0):
    backings = RegxExtract(backEx, "pass=%d" % round)
    dsLine = RegxExtract(dsEx, "pass=%d" % round)
    diskLine = RegxExtract(diskEx, "pass=%d" % round)
    round += 1

    expectedBackings = 80
    if len(backings) < expectedBackings:
        print 'found %d backings, less than the minimum expected %d' %\
            (len(backings), expectedBackings)
        fail = 1
        break

    expectedDs = 3
    if len(dsLine) != expectedDs:
        print 'found %d datastores, expected %d' %\
            (len(dsLine), expectedDs)
        for ds in dsLine:
            print ds
        fail = 1
        break

    flatDiskSize = {}
    flatDiskDatastore = {}
    for disk in diskLine:
        diskId = long(disk[1])
        diskMB = int(disk[2])
        diskDS = long(disk[4])
        flatDiskSize[diskId] = diskMB
        flatDiskDatastore[diskId] = diskDS

    lastDepth = 0
    cidToDatastore = {}
    cidToSize = {}
    for backing in backings:
        #print 'processing: ', backing
        backingCid   = long(backing[0])
        backingSize  = int(backing[1])
        backingDepth = int(backing[3])
        backingDs    = long(backing[4])
        backingDisk  = long(backing[5])

        # DrmDiskBackings must have ascending depth
        if (backingDepth != (lastDepth - 1)) and (lastDepth != 0):
            print 'backing %s has out of order files (round %d)' %\
                (backingCid, (round - 1))
            fail = 1
            lastDepth = backingDepth

        if fail != 0:
            break

        # Build various maps from cid...
        cidToSize[backingCid] = backingSize
        if cidToDatastore.has_key(backingCid):
            cidToDatastore[backingCid].add(backingDs)
        else:
            cidToDatastore[backingCid] = set([backingDs])

        # override disk MB sum in lieue of backing sums
        flatDiskSize[backingDisk] = 0

    if fail != 0:
        print 'Verify stopped at round %d due to failures' % (round - 1)
        break

    # Datastore used space must match the sum of all backings
    for ds in dsLine:
        dsId = long(ds[0])
        dsUsed = int (ds[2])
        dsSum = 0

        # space usage due to backing disks
        for (k,v) in cidToDatastore.items():
            if dsId in v:
                dsSum += cidToSize[k]

        # space usage due to flat disks
        for (k,v) in flatDiskDatastore.items():
            if v == dsId:
                dsSum += flatDiskSize[k]

        if dsSum != dsUsed:
            tmpSum = 0
            for (k,v) in sorted(cidToDatastore.iteritems(), key=lambda (k,v): (v,k)):
                if dsId in v:
                    tmpSum += cidToSize[k]
                    #print 'backing cid:%5s size:%5d sum:%6d' % (k, cidToSize[k], tmpSum)
            print 'datastore %d used %d != sum %d (round %d)' %\
                (dsId, dsUsed, dsSum, (round - 1))
            print 'output:', ds
            fail = 1
            break

    if fail != 0:
        print 'Verify stopped at round %d due to failures' % (round - 1)
        break
