#beginResultCriteria

# The powering-on InitPlaceHugeMin VMs cause the min of spare VMs to be 
# 20000 MHz. Even though the InitPlaceHugeMin VMs fail to power on, the spare 
# VMs still cause host admission to be violated. So skip checking the error
# messages.

modifiers["SkipHostAdmission"] = 1

# VMs with huge mins should fail to power on
hugemins = RegxExtract("vm:InitPlaceHugeMin(\d+) on host:(\S+)")
if hugemins:
	print "InitPlaceHugeMin VMs not expected to be powered on"
	fail = 1

# In pass 2 (when the first batch of VMs are powered on), the memory
# reservations of group "parent" should be scaled by a factor of
# (30720 - 80 * 2) / 30720 = 0.99, because the cluster capacity (30720 MB) 
# is less than the sum of "parent" reservation (30720 MB) and spare 
# reservations (80 * 2 = 160 MB).

if not RegxMatch("RepairRedGroup: group user, rsc mem, scaling reservations by 0.99", "pass==2"):
	print "Did not scale child reservations of pool 'user' by 0.99"
	fail = 1

# In the end, high shares VMs should be on DrmBigCpu, mid shares on DrmMidCpu and low shares on DrmSmallCpu

highshares = RegxExtract("vm:BusyHighShares2Cpu\d+ on host:(\S+)", "pass==7")
if len(highshares) != 5:
	print len(highshares), " BusyHighShares2Cpu VMs powered on, expecting 5"
	fail = 1
for host in highshares:
	if host.find('DrmBigCpu') == -1:
		print "BusyHighShares2Cpu VM on host ", host, ", expecting DrmBigCpu"
		fail = 1

midshares = RegxExtract("vm:BusyMidShares2Cpu\d+ on host:(\S+)", "pass==7")
if len(midshares) != 5:
	print len(midshares), " BusyMidShares2Cpu VMs powered on, expecting 5"
	fail = 1
for host in midshares:
	if host.find('DrmMidCpu') == -1:
		print "BusyMidShares2Cpu VM on host ", host, ", expecting DrmMidCpu"
		fail = 1

lowshares = RegxExtract("vm:BusyLowShares2Cpu\d+ on host:(\S+)", "pass==7")
if len(lowshares) != 5:
	print len(lowshares), " BusyLowShares2Cpu VMs powered on, expecting 5"
	fail = 1
for host in lowshares:
	if host.find('DrmSmallCpu') == -1:
		print "BusyLowShares2Cpu VM on host ", host, ", expecting DrmSmallCpu"
		fail = 1

#Ensure there is no host admission violation
if (not modifiers.has_key("SkipHostAdmission")) and\
   (RegxMatch(": host admission violated: ")):
   print "Host admission violated"
   fail = 1
#endResultCriteria
