#exclude default_hosts

numrounds=13

host DrmSmallCpu 1
host DrmSmallCpu 2 DRM_HOST_STANDBY

hostset hostset1 DrmSmallCpu1
hostset hostset2 DrmSmallCpu2
hostset hostset3 DrmSmallCpu3

vm UniIdle 1 user randhost
vm UniIdle 1 user delayplace hostset2
vm UniHighSharesBusy 2 user delayplace hostset3
vm UniIdle 2 user delayplace allHosts
vm UniHighSharesBusy 2 user delayplace allHosts

# UniIdle1 originally compat w/all hosts, running on only ok host DrmSmallCpu1.

# In the pass 2 placement of UniIdle2, the only available powered-on host
# DrmSmallCpu1 would be the attractive choice, except UniIdle2 is not
# compatible with that host, and hence won't be selected.
event:place UniIdle2 2 randhost

# At pass 3, user requests DrmSmallCpu1 enter maint, but UniIdle1 cannot be
# evacuated, since in pass 3, UniIdle1 became only compatible w/DrmSmallCpu1.
event:hostsetChange UniIdle1 3 hostset1
event:hostStateChange DrmSmallCpu1 3 DRM_HOST_GOING_DOWN

# In pass 4, UniIdle1 becomes incompatible with its current host,
# due to mandatory VM/host affinity rule being created; expect rec
# to correct the rule violation
event:hostsetChange UniIdle1 4 hostset2
event:hostStateChange DrmSmallCpu1 4 DRM_HOST_OK

# In pass 5, UniIdle1 becomes incompatible with its current host,
# due to mandatory VM/host affinity rule being created, but rule
# violation cannot be corrected; expect fault
event:hostsetChange UniIdle1 5 hostset1
event:hostStateChange DrmSmallCpu1 5 DRM_HOST_GOING_DOWN

# In pass 6, UniIdle1 becomes incompatible with its current host, due to
# soft VM/host affinity rule being created; but rule vio cannot be corrected
# because the needed host is in maint mode; expect fault.
event:hostsetChange UniIdle1 6 allHosts
event:hostsetChangeSoft UniIdle1 6 hostset1

# In pass 7, UniIdle1 is also incompatible w/its current host, but vio can be
# corrected; expect rec to do so which will include power-on DrmSmallCpu3.
event:hostsetChangeSoft UniIdle1 7 hostset3

# In pass 8, user requests DrmSmallCpu3 enter standby; DRS needs to drop
# UniIdle1 soft rule compliance to produce evacuation moves to support this.
event:hostStateChange DrmSmallCpu3 8 DRM_HOST_ENTER_STANDBY

# In pass 9, 2 high util VMs placed on hostSet3, engendering exit-standby
# of DrmSmallCpu3.  DRS ProposeActions can't correct UniIdle1 soft rule vio
# by moving it to DrmSmallCpu3 w/o worsening >100% util, so compliance dropped.
event:place UniHighSharesBusy3 9 randhost
event:place UniHighSharesBusy4 9 randhost

# In pass 10, reset soft rule for UniIdle1 so it is in compliance.
# Place UniIdle5 in compliance w/soft rule association w/DrmSmallCpu3.
event:hostsetChangeSoft UniIdle1 10 hostset2
event:hostsetChangeSoft UniIdle5 9 hostset3
event:place UniIdle5 10 randhost

# In pass 11, try to place UniIdle6 in compliance w/soft rule association w/
# DrmSmallCpu2; must drop soft rule to avoid anti-affin violation.
event:antiaffin UniIdle6 UniIdle1
event:hostsetChangeSoft UniIdle6 10 hostset2
event:place UniIdle6 11 randhost

# In pass 12, place UniHighSharesBusy7 which makes cluster demand > capacity.
# Soft rules are unilaterally dropped for this share-dominant regimen.
event:place UniHighSharesBusy7 12 randhost

# In pass 13, place UniHighSharesBusy8 while cluster demand > capacity.
# Soft rules would be unilaterally dropped for this share-dominant regimen,
# but advanced option used to disable dropping soft rules for overutilization
event:place UniHighSharesBusy8 13 randhost
event:setadvoption DropSoftVmHostRulesOverutilized 13 0

