# This test verifies whether the advanced option LimitVmsPerEsxHostPercent is
# honored by DRS power-on, load balancing and maintenance mode.
#
# It starts with 3 hosts and 3 VMs. Later it powers on 10 VMs in round 1 & round
# 2. The checker will verify the power-ons succeed and the constraint was not
# violated.
# Round 1: 8 VMs, 3 hosts & LimitVMsPerESXHostPercent = 10%
#          No host should have > ceil(8 / 3 * 110%) = 3 VMs each
#
# Round 2: 13 VMs, 3 hosts & LimitVMsPerESXHostPercent = 10%
#          No host should have > ceil(13 / 3 * 110%) = 5 VMs each
#
# In round 3, one of the bigger hosts was brought down to simulate maintenance
# mode.
# Round 3: 13 VMs, 2 hosts & LimitVMsPerESXHostPercent = 10%
#          No host should have > ceil(13 / 2 * 110%) = 8 VMs each
#
# In round 4, 2 more VMs are powered-on.
# No host should have > ceil(15 / 2 * 110%) = 9 VMs each
#
# Since one of the bigger hosts is brought down, because of the constraint there
# will be more contention in the smaller host. In round 5, the constraint is
# relaxed from 10% to 30%.
# Load balancing will do the best given the constraint (i.e., no host has more
# than 10 VMs each)

#exclude default_hosts

host TypicalHost 2
host DrmBigCpu 1
vm HighlyVariableLoad 12 user delayplace
vm BigMemWorkload 3 user randhost

numrounds=5
Power=0
NumUnreservedSlots=2
LimitVMsPerESXHostPercent=10

# Place 5 new VMs in Round 1
event:place HighlyVariableLoad1 1 randhost
event:place HighlyVariableLoad2 1 randhost
event:place HighlyVariableLoad3 1 randhost
event:place HighlyVariableLoad4 1 randhost
event:place HighlyVariableLoad5 1 randhost

# Place another 5 VMs in Round 2
event:place HighlyVariableLoad6  2 randhost
event:place HighlyVariableLoad7  2 randhost
event:place HighlyVariableLoad8  2 randhost
event:place HighlyVariableLoad9  2 randhost
event:place HighlyVariableLoad10 2 randhost

# Bring down a host in Round 3
event:hostStateChange TypicalHost1 3 DRM_HOST_GOING_DOWN

# Place remaining Vms in Round 4
event:place HighlyVariableLoad11 4 randhost
event:place HighlyVariableLoad12 4 randhost

event:setadvoption LimitVMsPerESXHostPercent 5 30
