###
### makefile to run basic drm test suite
###

include $(MAKEFILEDIR)/defs.mk

PYTHON_BASE = $(SRCROOT)/vpx/support/drm/python
PYTHON_PATH = $(PYTHON_BASE)

ifeq ($(OSTYPE), Win32)
PATH := $(VMACORE_LIB_DIR);$(PATH)
PATH := $(VMOMI_LIB_DIR);$(PATH)
else
# Path component separator on *nix is ':' not ';'
PATH := $(VMACORE_LIB_DIR):$(PATH)
PATH := $(VMOMI_LIB_DIR):$(PATH)
endif

DRMSUITE_PY = $(PYTHON_BASE)/drmsuite.py

## Test Options
##
##	These are useful options to override on the commandline when running
##	the tests, e.g. you could issue:
##
##		'make DRMTEST_LIST=file vpx-drmtest'
##
## where file is the prefix of the files in vpx/support/drm/tests.
## For e.g. spaceGrowth
##
DRMTEST_INPUT = $(SRCROOT)/vpx/support/drm/tests
DRMTEST_OUTPUT = $(VMWAREBUILDDIR)/drmtest
DRMTEST_LOGLEVEL = 3
DRMTEST_THRESH = 3
DRMTEST_HOSTS = setup/default_hosts
DRMTEST_DEFAULTS = setup/default_opts
DRMTEST_VM_CONFIG = $(DRMTEST_INPUT)/setup/templates.ini
DRMTEST_ROUNDS = 12
DRMTEST_INTERVAL = 300
ifdef DRMTEST_OPTS
DRMTEST_OPTS_STR := -O $(DRMTEST_OPTS)
endif
ifdef DRMTEST_SEED
DRMTEST_SEED_STR := --randseed $(DRMTEST_SEED)
endif
DRMTEST_RECORD = 0
DRMTEST_COMPARE = 1
DRMTEST_POWER = 1
DRMTEST_STORAGEF = 0
ifdef DRMTEST_STORAGE
DRMTEST_STORAGEF = 1
endif
# can specify a specific set of tests by defining DRM_TEST_LIST
# explicitly on the commandline
include $(DRMTEST_INPUT)/test_list.mk

DRMTEST_LIST_STR := $(addsuffix .tst,$(sort $(DRMTEST_LIST:%=$(DRMTEST_INPUT)/%)))
DRMTEST_HOSTS_STR := $(DRMTEST_INPUT)/$(DRMTEST_HOSTS).tst
DRMTEST_DEFAULTS_STR := $(DRMTEST_INPUT)/$(DRMTEST_DEFAULTS).tst

DRMSUITE_CMD = $(PYTHON) $(DRMSUITE_PY) -d $(VPX_EXE) \
		  -H $(DRMTEST_HOSTS_STR) -v $(DRMTEST_VM_CONFIG) \
		  -o $(DRMTEST_OUTPUT) -I $(DRMTEST_DEFAULTS_STR) \
                  -c $(DRMTEST_COMPARE) -a $(DRMTEST_RECORD) \
                  -m $(DRMTEST_THRESH)\
                  -P $(DRMTEST_POWER)\
                  -g $(DRMTEST_STORAGEF)\
                  -E $(OSTYPE) \
	          -l $(DRMTEST_LOGLEVEL) $(DRMTEST_OPTS_STR) $(DRMTEST_SEED_STR)

ifdef DRMTEST_USE_VPX
# XXX - Linux
VPX_EXE = $(VMWAREBUILDDIR)/vpxd/vpxd.exe
PYTHON_PATH := $(PYTHON_PATH);$(VMWAREBUILDDIR)/vim/lib/pyVmacore
PYTHON_PATH := $(PYTHON_PATH);$(VMWAREBUILDDIR)/../../vmodl
PATH := $(PATH);$(VMWAREBUILDDIR)/vim/hostd/types
DRMSUITE_CMD += -L "$(DRMTEST_LBASE)" -B "$(DRMTEST_SBASE)"
else
ifeq ($(OSTYPE), Win32)
VPX_EXE = $(VMWAREBUILDDIR)/dummydrm/dummydrm.exe
else
VPX_EXE = $(VMWAREBUILDDIR)/dummydrm/dummydrm
endif
DRMSUITE_CMD += --runsim $(DRMTEST_ROUNDS) --interval $(DRMTEST_INTERVAL)
endif

ifeq ($(OSTYPE), Win32)
build: $(PHONY) $(DRMTEST_OUTPUT)
	$(DRMSUITE_CMD) $(DRMTEST_LIST_STR)
else
build: $(PHONY) $(DRMTEST_OUTPUT)
	LD_LIBRARY_PATH=$(CC_LIBDIR):$(GOBUILD_VIMBASE_ROOT)/x86/lib:$(LIBEXPAT_LIBPATH):$(VMACORE_LIB_DIR):$(VMOMI_LIB_DIR): $(DRMSUITE_CMD) $(DRMTEST_LIST_STR)
endif

$(DRMTEST_OUTPUT):
	mkdir $(DRMTEST_OUTPUT)
